# Generated from the eo-protocol XML specification.
#
# This file should not be modified.
# Changes will be lost when code is regenerated.

from enum import IntEnum
from ....protocol_enum_meta import ProtocolEnumMeta

class PriestReply(IntEnum, metaclass=ProtocolEnumMeta):
    """
    Reply code sent with PRIEST_REPLY packet
    """
    NotDressed = 1
    LowLevel = 2
    PartnerNotPresent = 3
    PartnerNotDressed = 4
    Busy = 5
    DoYou = 6
    PartnerAlreadyMarried = 7
    NoPermission = 8



import argparse
import os
import re
import shutil
import keepachangelog
from pathlib import Path


def change_working_directory():
    working_dir = str(Path(__file__).parent.resolve())
    os.chdir(working_dir)


def validate_version(version):
    if not re.match(r'^\d+\.\d+\.\d+([a-z]+\d+)?$', version):
        raise RuntimeError('invalid SemVer version: expected format "major.minor.patch[label]"')


def validate_git_status():
    if shutil.which('git') is None:
        raise RuntimeError('git could not be found')
    if os.popen('git status --porcelain').read():
        raise RuntimeError('dirty working tree: commit changes and try again')


def validate_tag(tag):
    if os.popen(f'git tag -l {tag}').read():
        raise RuntimeError(f'tag {tag} already exists')


def write_about_file(version):
    with open('src/eolib/__about__.py', 'w') as about:
        about.write(f'__version__ = "{version}"\n')


def bump_changelog(version):
    keepachangelog.release('CHANGELOG.md', version)


def commit(tag):
    if (
        os.system('git add .')
        or os.system(f'git commit -m "Release {tag}"')
        or os.system(f'git tag {tag} HEAD')
    ):
        raise RuntimeError('failed to commit/tag changes')


def prepare_release(version):
    tag = f'v{version}'

    change_working_directory()

    validate_version(version)
    validate_git_status()
    validate_tag(tag)

    write_about_file(version)
    bump_changelog(version)
    commit(tag)

    print(tag)


parser = argparse.ArgumentParser(description='Prepare release')
parser.add_argument(
    'version', type=str, help='New version in SemVer format (e.g., major.minor.patch[label])'
)
version = parser.parse_args().version

try:
    prepare_release(version)
except RuntimeError as e:
    print(str(e))
    exit(1)



from protocol_code_generator.type.basic_type import BasicType


class IntegerType(BasicType):
    def __init__(self, name, size):
        self._name = name
        self._size = size

    @property
    def name(self):
        return self._name

    @property
    def fixed_size(self):
        return self._size

    @property
    def bounded(self):
        return True



# Generated from the eo-protocol XML specification.
#
# This file should not be modified.
# Changes will be lost when code is regenerated.

from __future__ import annotations
from collections.abc import Iterable
from .map_drain_damage_other import MapDrainDamageOther
from ..packet_family import PacketFamily
from ..packet_action import PacketAction
from ....serialization_error import SerializationError
from ....net.packet import Packet
from .....data.eo_writer import EoWriter
from .....data.eo_reader import EoReader

class EffectTargetOtherServerPacket(Packet):
    """
    Map drain damage
    """
    _byte_size: int = 0
    _damage: int
    _hp: int
    _max_hp: int
    _others: tuple[MapDrainDamageOther, ...]

    def __init__(self, *, damage: int, hp: int, max_hp: int, others: Iterable[MapDrainDamageOther]):
        """
        Create a new instance of EffectTargetOtherServerPacket.

        Args:
            damage (int): (Value range is 0-64008.)
            hp (int): (Value range is 0-64008.)
            max_hp (int): (Value range is 0-64008.)
            others (Iterable[MapDrainDamageOther]): 
        """
        self._damage = damage
        self._hp = hp
        self._max_hp = max_hp
        self._others = tuple(others)

    @property
    def byte_size(self) -> int:
        """
        Returns the size of the data that this was deserialized from.

        Returns:
            int: The size of the data that this was deserialized from.
        """
        return self._byte_size

    @property
    def damage(self) -> int:
        return self._damage

    @property
    def hp(self) -> int:
        return self._hp

    @property
    def max_hp(self) -> int:
        return self._max_hp

    @property
    def others(self) -> tuple[MapDrainDamageOther, ...]:
        return self._others

    @staticmethod
    def family() -> PacketFamily:
        """
        Returns the packet family associated with this packet.

        Returns:
            PacketFamily: The packet family associated with this packet.
        """
        return PacketFamily.Effect

    @staticmethod
    def action() -> PacketAction:
        """
        Returns the packet action associated with this packet.

        Returns:
            PacketAction: The packet action associated with this packet.
        """
        return PacketAction.TargetOther

    def write(self, writer):
        """
        Serializes and writes this packet to the provided EoWriter.

        Args:
            writer (EoWriter): the writer that this packet will be written to.
        """
        EffectTargetOtherServerPacket.serialize(writer, self)

    @staticmethod
    def serialize(writer: EoWriter, data: "EffectTargetOtherServerPacket") -> None:
        """
        Serializes an instance of `EffectTargetOtherServerPacket` to the provided `EoWriter`.

        Args:
            writer (EoWriter): The writer that the data will be serialized to.
            data (EffectTargetOtherServerPacket): The data to serialize.
        """
        old_string_sanitization_mode: bool = writer.string_sanitization_mode
        try:
            if data._damage is None:
                raise SerializationError("damage must be provided.")
            writer.add_short(data._damage)
            if data._hp is None:
                raise SerializationError("hp must be provided.")
            writer.add_short(data._hp)
            if data._max_hp is None:
                raise SerializationError("max_hp must be provided.")
            writer.add_short(data._max_hp)
            if data._others is None:
                raise SerializationError("others must be provided.")
            for i in range(len(data._others)):
                MapDrainDamageOther.serialize(writer, data._others[i])
        finally:
            writer.string_sanitization_mode = old_string_sanitization_mode

    @staticmethod
    def deserialize(reader: EoReader) -> "EffectTargetOtherServerPacket":
        """
        Deserializes an instance of `EffectTargetOtherServerPacket` from the provided `EoReader`.

        Args:
            reader (EoReader): The writer that the data will be serialized to.

        Returns:
            EffectTargetOtherServerPacket: The data to serialize.
        """
        old_chunked_reading_mode: bool = reader.chunked_reading_mode
        try:
            reader_start_position: int = reader.position
            damage = reader.get_short()
            hp = reader.get_short()
            max_hp = reader.get_short()
            others_length = int(reader.remaining / 5)
            others = []
            for i in range(others_length):
                others.append(MapDrainDamageOther.deserialize(reader))
            result = EffectTargetOtherServerPacket(damage=damage, hp=hp, max_hp=max_hp, others=others)
            result._byte_size = reader.position - reader_start_position
            return result
        finally:
            reader.chunked_reading_mode = old_chunked_reading_mode

    def __repr__(self):
        return f"EffectTargetOtherServerPacket(byte_size={repr(self._byte_size)}, damage={repr(self._damage)}, hp={repr(self._hp)}, max_hp={repr(self._max_hp)}, others={repr(self._others)})"



# Generated from the eo-protocol XML specification.
#
# This file should not be modified.
# Changes will be lost when code is regenerated.

from typing import Union
from .pub_file import PubFile
from .players_list_friends import PlayersListFriends
from .players_list import PlayersList
from .map_file import MapFile
from .init_reply import InitReply
from .init_ban_type import InitBanType
from ..version import Version
from ..packet_family import PacketFamily
from ..packet_action import PacketAction
from ....serialization_error import SerializationError
from ....net.packet import Packet
from .....data.eo_writer import EoWriter
from .....data.eo_reader import EoReader

class InitInitServerPacket(Packet):
    """
    Reply to connection initialization and requests for unencrypted data.
    This packet is unencrypted.
    """
    _byte_size: int = 0
    _reply_code: InitReply
    _reply_code_data: 'InitInitServerPacket.ReplyCodeData'

    def __init__(self, *, reply_code: InitReply, reply_code_data: 'InitInitServerPacket.ReplyCodeData' = None):
        """
        Create a new instance of InitInitServerPacket.

        Args:
            reply_code (InitReply): 
            reply_code_data (InitInitServerPacket.ReplyCodeData): Data associated with the `reply_code` field.
        """
        self._reply_code = reply_code
        self._reply_code_data = reply_code_data

    @property
    def byte_size(self) -> int:
        """
        Returns the size of the data that this was deserialized from.

        Returns:
            int: The size of the data that this was deserialized from.
        """
        return self._byte_size

    @property
    def reply_code(self) -> InitReply:
        return self._reply_code

    @property
    def reply_code_data(self) -> 'InitInitServerPacket.ReplyCodeData':
        """
        InitInitServerPacket.ReplyCodeData: Data associated with the `reply_code` field.
        """
        return self._reply_code_data

    @staticmethod
    def family() -> PacketFamily:
        """
        Returns the packet family associated with this packet.

        Returns:
            PacketFamily: The packet family associated with this packet.
        """
        return PacketFamily.Init

    @staticmethod
    def action() -> PacketAction:
        """
        Returns the packet action associated with this packet.

        Returns:
            PacketAction: The packet action associated with this packet.
        """
        return PacketAction.Init

    def write(self, writer):
        """
        Serializes and writes this packet to the provided EoWriter.

        Args:
            writer (EoWriter): the writer that this packet will be written to.
        """
        InitInitServerPacket.serialize(writer, self)

    @staticmethod
    def serialize(writer: EoWriter, data: "InitInitServerPacket") -> None:
        """
        Serializes an instance of `InitInitServerPacket` to the provided `EoWriter`.

        Args:
            writer (EoWriter): The writer that the data will be serialized to.
            data (InitInitServerPacket): The data to serialize.
        """
        old_string_sanitization_mode: bool = writer.string_sanitization_mode
        try:
            if data._reply_code is None:
                raise SerializationError("reply_code must be provided.")
            writer.add_byte(int(data._reply_code))
            if data._reply_code == InitReply.OutOfDate:
                if not isinstance(data._reply_code_data, InitInitServerPacket.ReplyCodeDataOutOfDate):
                    raise SerializationError("Expected reply_code_data to be type InitInitServerPacket.ReplyCodeDataOutOfDate for reply_code " + InitReply(data._reply_code).name + ".")
                InitInitServerPacket.ReplyCodeDataOutOfDate.serialize(writer, data._reply_code_data)
            elif data._reply_code == InitReply.Ok:
                if not isinstance(data._reply_code_data, InitInitServerPacket.ReplyCodeDataOk):
                    raise SerializationError("Expected reply_code_data to be type InitInitServerPacket.ReplyCodeDataOk for reply_code " + InitReply(data._reply_code).name + ".")
                InitInitServerPacket.ReplyCodeDataOk.serialize(writer, data._reply_code_data)
            elif data._reply_code == InitReply.Banned:
                if not isinstance(data._reply_code_data, InitInitServerPacket.ReplyCodeDataBanned):
                    raise SerializationError("Expected reply_code_data to be type InitInitServerPacket.ReplyCodeDataBanned for reply_code " + InitReply(data._reply_code).name + ".")
                InitInitServerPacket.ReplyCodeDataBanned.serialize(writer, data._reply_code_data)
            elif data._reply_code == InitReply.WarpMap:
                if not isinstance(data._reply_code_data, InitInitServerPacket.ReplyCodeDataWarpMap):
                    raise SerializationError("Expected reply_code_data to be type InitInitServerPacket.ReplyCodeDataWarpMap for reply_code " + InitReply(data._reply_code).name + ".")
                InitInitServerPacket.ReplyCodeDataWarpMap.serialize(writer, data._reply_code_data)
            elif data._reply_code == InitReply.FileEmf:
                if not isinstance(data._reply_code_data, InitInitServerPacket.ReplyCodeDataFileEmf):
                    raise SerializationError("Expected reply_code_data to be type InitInitServerPacket.ReplyCodeDataFileEmf for reply_code " + InitReply(data._reply_code).name + ".")
                InitInitServerPacket.ReplyCodeDataFileEmf.serialize(writer, data._reply_code_data)
            elif data._reply_code == InitReply.FileEif:
                if not isinstance(data._reply_code_data, InitInitServerPacket.ReplyCodeDataFileEif):
                    raise SerializationError("Expected reply_code_data to be type InitInitServerPacket.ReplyCodeDataFileEif for reply_code " + InitReply(data._reply_code).name + ".")
                InitInitServerPacket.ReplyCodeDataFileEif.serialize(writer, data._reply_code_data)
            elif data._reply_code == InitReply.FileEnf:
                if not isinstance(data._reply_code_data, InitInitServerPacket.ReplyCodeDataFileEnf):
                    raise SerializationError("Expected reply_code_data to be type InitInitServerPacket.ReplyCodeDataFileEnf for reply_code " + InitReply(data._reply_code).name + ".")
                InitInitServerPacket.ReplyCodeDataFileEnf.serialize(writer, data._reply_code_data)
            elif data._reply_code == InitReply.FileEsf:
                if not isinstance(data._reply_code_data, InitInitServerPacket.ReplyCodeDataFileEsf):
                    raise SerializationError("Expected reply_code_data to be type InitInitServerPacket.ReplyCodeDataFileEsf for reply_code " + InitReply(data._reply_code).name + ".")
                InitInitServerPacket.ReplyCodeDataFileEsf.serialize(writer, data._reply_code_data)
            elif data._reply_code == InitReply.FileEcf:
                if not isinstance(data._reply_code_data, InitInitServerPacket.ReplyCodeDataFileEcf):
                    raise SerializationError("Expected reply_code_data to be type InitInitServerPacket.ReplyCodeDataFileEcf for reply_code " + InitReply(data._reply_code).name + ".")
                InitInitServerPacket.ReplyCodeDataFileEcf.serialize(writer, data._reply_code_data)
            elif data._reply_code == InitReply.MapMutation:
                if not isinstance(data._reply_code_data, InitInitServerPacket.ReplyCodeDataMapMutation):
                    raise SerializationError("Expected reply_code_data to be type InitInitServerPacket.ReplyCodeDataMapMutation for reply_code " + InitReply(data._reply_code).name + ".")
                InitInitServerPacket.ReplyCodeDataMapMutation.serialize(writer, data._reply_code_data)
            elif data._reply_code == InitReply.PlayersList:
                if not isinstance(data._reply_code_data, InitInitServerPacket.ReplyCodeDataPlayersList):
                    raise SerializationError("Expected reply_code_data to be type InitInitServerPacket.ReplyCodeDataPlayersList for reply_code " + InitReply(data._reply_code).name + ".")
                InitInitServerPacket.ReplyCodeDataPlayersList.serialize(writer, data._reply_code_data)
            elif data._reply_code == InitReply.PlayersListFriends:
                if not isinstance(data._reply_code_data, InitInitServerPacket.ReplyCodeDataPlayersListFriends):
                    raise SerializationError("Expected reply_code_data to be type InitInitServerPacket.ReplyCodeDataPlayersListFriends for reply_code " + InitReply(data._reply_code).name + ".")
                InitInitServerPacket.ReplyCodeDataPlayersListFriends.serialize(writer, data._reply_code_data)
        finally:
            writer.string_sanitization_mode = old_string_sanitization_mode

    @staticmethod
    def deserialize(reader: EoReader) -> "InitInitServerPacket":
        """
        Deserializes an instance of `InitInitServerPacket` from the provided `EoReader`.

        Args:
            reader (EoReader): The writer that the data will be serialized to.

        Returns:
            InitInitServerPacket: The data to serialize.
        """
        old_chunked_reading_mode: bool = reader.chunked_reading_mode
        try:
            reader_start_position: int = reader.position
            reply_code = InitReply(reader.get_byte())
            reply_code_data: InitInitServerPacket.ReplyCodeData = None
            if reply_code == InitReply.OutOfDate:
                reply_code_data = InitInitServerPacket.ReplyCodeDataOutOfDate.deserialize(reader)
            elif reply_code == InitReply.Ok:
                reply_code_data = InitInitServerPacket.ReplyCodeDataOk.deserialize(reader)
            elif reply_code == InitReply.Banned:
                reply_code_data = InitInitServerPacket.ReplyCodeDataBanned.deserialize(reader)
            elif reply_code == InitReply.WarpMap:
                reply_code_data = InitInitServerPacket.ReplyCodeDataWarpMap.deserialize(reader)
            elif reply_code == InitReply.FileEmf:
                reply_code_data = InitInitServerPacket.ReplyCodeDataFileEmf.deserialize(reader)
            elif reply_code == InitReply.FileEif:
                reply_code_data = InitInitServerPacket.ReplyCodeDataFileEif.deserialize(reader)
            elif reply_code == InitReply.FileEnf:
                reply_code_data = InitInitServerPacket.ReplyCodeDataFileEnf.deserialize(reader)
            elif reply_code == InitReply.FileEsf:
                reply_code_data = InitInitServerPacket.ReplyCodeDataFileEsf.deserialize(reader)
            elif reply_code == InitReply.FileEcf:
                reply_code_data = InitInitServerPacket.ReplyCodeDataFileEcf.deserialize(reader)
            elif reply_code == InitReply.MapMutation:
                reply_code_data = InitInitServerPacket.ReplyCodeDataMapMutation.deserialize(reader)
            elif reply_code == InitReply.PlayersList:
                reply_code_data = InitInitServerPacket.ReplyCodeDataPlayersList.deserialize(reader)
            elif reply_code == InitReply.PlayersListFriends:
                reply_code_data = InitInitServerPacket.ReplyCodeDataPlayersListFriends.deserialize(reader)
            result = InitInitServerPacket(reply_code=reply_code, reply_code_data=reply_code_data)
            result._byte_size = reader.position - reader_start_position
            return result
        finally:
            reader.chunked_reading_mode = old_chunked_reading_mode

    def __repr__(self):
        return f"InitInitServerPacket(byte_size={repr(self._byte_size)}, reply_code={repr(self._reply_code)}, reply_code_data={repr(self._reply_code_data)})"

    ReplyCodeData = Union['InitInitServerPacket.ReplyCodeDataOutOfDate', 'InitInitServerPacket.ReplyCodeDataOk', 'InitInitServerPacket.ReplyCodeDataBanned', 'InitInitServerPacket.ReplyCodeDataWarpMap', 'InitInitServerPacket.ReplyCodeDataFileEmf', 'InitInitServerPacket.ReplyCodeDataFileEif', 'InitInitServerPacket.ReplyCodeDataFileEnf', 'InitInitServerPacket.ReplyCodeDataFileEsf', 'InitInitServerPacket.ReplyCodeDataFileEcf', 'InitInitServerPacket.ReplyCodeDataMapMutation', 'InitInitServerPacket.ReplyCodeDataPlayersList', 'InitInitServerPacket.ReplyCodeDataPlayersListFriends', None]
    """
    Data associated with different values of the `reply_code` field.
    """

    class ReplyCodeDataOutOfDate:
        """
        Data associated with reply_code value InitReply.OutOfDate
        """
        _byte_size: int = 0
        _version: Version

        def __init__(self, *, version: Version):
            """
            Create a new instance of InitInitServerPacket.ReplyCodeDataOutOfDate.

            Args:
                version (Version): 
            """
            self._version = version

        @property
        def byte_size(self) -> int:
            """
            Returns the size of the data that this was deserialized from.

            Returns:
                int: The size of the data that this was deserialized from.
            """
            return self._byte_size

        @property
        def version(self) -> Version:
            return self._version

        @staticmethod
        def serialize(writer: EoWriter, data: "InitInitServerPacket.ReplyCodeDataOutOfDate") -> None:
            """
            Serializes an instance of `InitInitServerPacket.ReplyCodeDataOutOfDate` to the provided `EoWriter`.

            Args:
                writer (EoWriter): The writer that the data will be serialized to.
                data (InitInitServerPacket.ReplyCodeDataOutOfDate): The data to serialize.
            """
            old_string_sanitization_mode: bool = writer.string_sanitization_mode
            try:
                if data._version is None:
                    raise SerializationError("version must be provided.")
                Version.serialize(writer, data._version)
            finally:
                writer.string_sanitization_mode = old_string_sanitization_mode

        @staticmethod
        def deserialize(reader: EoReader) -> "InitInitServerPacket.ReplyCodeDataOutOfDate":
            """
            Deserializes an instance of `InitInitServerPacket.ReplyCodeDataOutOfDate` from the provided `EoReader`.

            Args:
                reader (EoReader): The writer that the data will be serialized to.

            Returns:
                InitInitServerPacket.ReplyCodeDataOutOfDate: The data to serialize.
            """
            old_chunked_reading_mode: bool = reader.chunked_reading_mode
            try:
                reader_start_position: int = reader.position
                version = Version.deserialize(reader)
                result = InitInitServerPacket.ReplyCodeDataOutOfDate(version=version)
                result._byte_size = reader.position - reader_start_position
                return result
            finally:
                reader.chunked_reading_mode = old_chunked_reading_mode

        def __repr__(self):
            return f"InitInitServerPacket.ReplyCodeDataOutOfDate(byte_size={repr(self._byte_size)}, version={repr(self._version)})"

    class ReplyCodeDataOk:
        """
        Data associated with reply_code value InitReply.Ok
        """
        _byte_size: int = 0
        _seq1: int
        _seq2: int
        _server_encryption_multiple: int
        _client_encryption_multiple: int
        _player_id: int
        _challenge_response: int

        def __init__(self, *, seq1: int, seq2: int, server_encryption_multiple: int, client_encryption_multiple: int, player_id: int, challenge_response: int):
            """
            Create a new instance of InitInitServerPacket.ReplyCodeDataOk.

            Args:
                seq1 (int): (Value range is 0-255.)
                seq2 (int): (Value range is 0-255.)
                server_encryption_multiple (int): (Value range is 0-255.)
                client_encryption_multiple (int): (Value range is 0-255.)
                player_id (int): (Value range is 0-64008.)
                challenge_response (int): (Value range is 0-16194276.)
            """
            self._seq1 = seq1
            self._seq2 = seq2
            self._server_encryption_multiple = server_encryption_multiple
            self._client_encryption_multiple = client_encryption_multiple
            self._player_id = player_id
            self._challenge_response = challenge_response

        @property
        def byte_size(self) -> int:
            """
            Returns the size of the data that this was deserialized from.

            Returns:
                int: The size of the data that this was deserialized from.
            """
            return self._byte_size

        @property
        def seq1(self) -> int:
            return self._seq1

        @property
        def seq2(self) -> int:
            return self._seq2

        @property
        def server_encryption_multiple(self) -> int:
            return self._server_encryption_multiple

        @property
        def client_encryption_multiple(self) -> int:
            return self._client_encryption_multiple

        @property
        def player_id(self) -> int:
            return self._player_id

        @property
        def challenge_response(self) -> int:
            return self._challenge_response

        @staticmethod
        def serialize(writer: EoWriter, data: "InitInitServerPacket.ReplyCodeDataOk") -> None:
            """
            Serializes an instance of `InitInitServerPacket.ReplyCodeDataOk` to the provided `EoWriter`.

            Args:
                writer (EoWriter): The writer that the data will be serialized to.
                data (InitInitServerPacket.ReplyCodeDataOk): The data to serialize.
            """
            old_string_sanitization_mode: bool = writer.string_sanitization_mode
            try:
                if data._seq1 is None:
                    raise SerializationError("seq1 must be provided.")
                writer.add_byte(data._seq1)
                if data._seq2 is None:
                    raise SerializationError("seq2 must be provided.")
                writer.add_byte(data._seq2)
                if data._server_encryption_multiple is None:
                    raise SerializationError("server_encryption_multiple must be provided.")
                writer.add_byte(data._server_encryption_multiple)
                if data._client_encryption_multiple is None:
                    raise SerializationError("client_encryption_multiple must be provided.")
                writer.add_byte(data._client_encryption_multiple)
                if data._player_id is None:
                    raise SerializationError("player_id must be provided.")
                writer.add_short(data._player_id)
                if data._challenge_response is None:
                    raise SerializationError("challenge_response must be provided.")
                writer.add_three(data._challenge_response)
            finally:
                writer.string_sanitization_mode = old_string_sanitization_mode

        @staticmethod
        def deserialize(reader: EoReader) -> "InitInitServerPacket.ReplyCodeDataOk":
            """
            Deserializes an instance of `InitInitServerPacket.ReplyCodeDataOk` from the provided `EoReader`.

            Args:
                reader (EoReader): The writer that the data will be serialized to.

            Returns:
                InitInitServerPacket.ReplyCodeDataOk: The data to serialize.
            """
            old_chunked_reading_mode: bool = reader.chunked_reading_mode
            try:
                reader_start_position: int = reader.position
                seq1 = reader.get_byte()
                seq2 = reader.get_byte()
                server_encryption_multiple = reader.get_byte()
                client_encryption_multiple = reader.get_byte()
                player_id = reader.get_short()
                challenge_response = reader.get_three()
                result = InitInitServerPacket.ReplyCodeDataOk(seq1=seq1, seq2=seq2, server_encryption_multiple=server_encryption_multiple, client_encryption_multiple=client_encryption_multiple, player_id=player_id, challenge_response=challenge_response)
                result._byte_size = reader.position - reader_start_position
                return result
            finally:
                reader.chunked_reading_mode = old_chunked_reading_mode

        def __repr__(self):
            return f"InitInitServerPacket.ReplyCodeDataOk(byte_size={repr(self._byte_size)}, seq1={repr(self._seq1)}, seq2={repr(self._seq2)}, server_encryption_multiple={repr(self._server_encryption_multiple)}, client_encryption_multiple={repr(self._client_encryption_multiple)}, player_id={repr(self._player_id)}, challenge_response={repr(self._challenge_response)})"

    class ReplyCodeDataBanned:
        """
        Data associated with reply_code value InitReply.Banned
        """
        _byte_size: int = 0
        _ban_type: InitBanType
        _ban_type_data: 'InitInitServerPacket.ReplyCodeDataBanned.BanTypeData'

        def __init__(self, *, ban_type: InitBanType, ban_type_data: 'InitInitServerPacket.ReplyCodeDataBanned.BanTypeData' = None):
            """
            Create a new instance of InitInitServerPacket.ReplyCodeDataBanned.

            Args:
                ban_type (InitBanType): 
                ban_type_data (InitInitServerPacket.ReplyCodeDataBanned.BanTypeData): Data associated with the `ban_type` field.
            """
            self._ban_type = ban_type
            self._ban_type_data = ban_type_data

        @property
        def byte_size(self) -> int:
            """
            Returns the size of the data that this was deserialized from.

            Returns:
                int: The size of the data that this was deserialized from.
            """
            return self._byte_size

        @property
        def ban_type(self) -> InitBanType:
            return self._ban_type

        @property
        def ban_type_data(self) -> 'InitInitServerPacket.ReplyCodeDataBanned.BanTypeData':
            """
            InitInitServerPacket.ReplyCodeDataBanned.BanTypeData: Data associated with the `ban_type` field.
            """
            return self._ban_type_data

        @staticmethod
        def serialize(writer: EoWriter, data: "InitInitServerPacket.ReplyCodeDataBanned") -> None:
            """
            Serializes an instance of `InitInitServerPacket.ReplyCodeDataBanned` to the provided `EoWriter`.

            Args:
                writer (EoWriter): The writer that the data will be serialized to.
                data (InitInitServerPacket.ReplyCodeDataBanned): The data to serialize.
            """
            old_string_sanitization_mode: bool = writer.string_sanitization_mode
            try:
                if data._ban_type is None:
                    raise SerializationError("ban_type must be provided.")
                writer.add_byte(int(data._ban_type))
                if data._ban_type == 0:
                    if not isinstance(data._ban_type_data, InitInitServerPacket.ReplyCodeDataBanned.BanTypeData0):
                        raise SerializationError("Expected ban_type_data to be type InitInitServerPacket.ReplyCodeDataBanned.BanTypeData0 for ban_type " + InitBanType(data._ban_type).name + ".")
                    InitInitServerPacket.ReplyCodeDataBanned.BanTypeData0.serialize(writer, data._ban_type_data)
                elif data._ban_type == InitBanType.Temporary:
                    if not isinstance(data._ban_type_data, InitInitServerPacket.ReplyCodeDataBanned.BanTypeDataTemporary):
                        raise SerializationError("Expected ban_type_data to be type InitInitServerPacket.ReplyCodeDataBanned.BanTypeDataTemporary for ban_type " + InitBanType(data._ban_type).name + ".")
                    InitInitServerPacket.ReplyCodeDataBanned.BanTypeDataTemporary.serialize(writer, data._ban_type_data)
            finally:
                writer.string_sanitization_mode = old_string_sanitization_mode

        @staticmethod
        def deserialize(reader: EoReader) -> "InitInitServerPacket.ReplyCodeDataBanned":
            """
            Deserializes an instance of `InitInitServerPacket.ReplyCodeDataBanned` from the provided `EoReader`.

            Args:
                reader (EoReader): The writer that the data will be serialized to.

            Returns:
                InitInitServerPacket.ReplyCodeDataBanned: The data to serialize.
            """
            old_chunked_reading_mode: bool = reader.chunked_reading_mode
            try:
                reader_start_position: int = reader.position
                ban_type = InitBanType(reader.get_byte())
                ban_type_data: InitInitServerPacket.ReplyCodeDataBanned.BanTypeData = None
                if ban_type == 0:
                    ban_type_data = InitInitServerPacket.ReplyCodeDataBanned.BanTypeData0.deserialize(reader)
                elif ban_type == InitBanType.Temporary:
                    ban_type_data = InitInitServerPacket.ReplyCodeDataBanned.BanTypeDataTemporary.deserialize(reader)
                result = InitInitServerPacket.ReplyCodeDataBanned(ban_type=ban_type, ban_type_data=ban_type_data)
                result._byte_size = reader.position - reader_start_position
                return result
            finally:
                reader.chunked_reading_mode = old_chunked_reading_mode

        def __repr__(self):
            return f"InitInitServerPacket.ReplyCodeDataBanned(byte_size={repr(self._byte_size)}, ban_type={repr(self._ban_type)}, ban_type_data={repr(self._ban_type_data)})"

        BanTypeData = Union['InitInitServerPacket.ReplyCodeDataBanned.BanTypeData0', 'InitInitServerPacket.ReplyCodeDataBanned.BanTypeDataTemporary', None]
        """
        Data associated with different values of the `ban_type` field.
        """

        class BanTypeData0:
            """
            Data associated with ban_type value 0

            The official client treats any value below 2 as a temporary ban.
            The official server sends 1, but some game server implementations
            erroneously send 0.
            """
            _byte_size: int = 0
            _minutes_remaining: int

            def __init__(self, *, minutes_remaining: int):
                """
                Create a new instance of InitInitServerPacket.ReplyCodeDataBanned.BanTypeData0.

                Args:
                    minutes_remaining (int): (Value range is 0-255.)
                """
                self._minutes_remaining = minutes_remaining

            @property
            def byte_size(self) -> int:
                """
                Returns the size of the data that this was deserialized from.

                Returns:
                    int: The size of the data that this was deserialized from.
                """
                return self._byte_size

            @property
            def minutes_remaining(self) -> int:
                return self._minutes_remaining

            @staticmethod
            def serialize(writer: EoWriter, data: "InitInitServerPacket.ReplyCodeDataBanned.BanTypeData0") -> None:
                """
                Serializes an instance of `InitInitServerPacket.ReplyCodeDataBanned.BanTypeData0` to the provided `EoWriter`.

                Args:
                    writer (EoWriter): The writer that the data will be serialized to.
                    data (InitInitServerPacket.ReplyCodeDataBanned.BanTypeData0): The data to serialize.
                """
                old_string_sanitization_mode: bool = writer.string_sanitization_mode
                try:
                    if data._minutes_remaining is None:
                        raise SerializationError("minutes_remaining must be provided.")
                    writer.add_byte(data._minutes_remaining)
                finally:
                    writer.string_sanitization_mode = old_string_sanitization_mode

            @staticmethod
            def deserialize(reader: EoReader) -> "InitInitServerPacket.ReplyCodeDataBanned.BanTypeData0":
                """
                Deserializes an instance of `InitInitServerPacket.ReplyCodeDataBanned.BanTypeData0` from the provided `EoReader`.

                Args:
                    reader (EoReader): The writer that the data will be serialized to.

                Returns:
                    InitInitServerPacket.ReplyCodeDataBanned.BanTypeData0: The data to serialize.
                """
                old_chunked_reading_mode: bool = reader.chunked_reading_mode
                try:
                    reader_start_position: int = reader.position
                    minutes_remaining = reader.get_byte()
                    result = InitInitServerPacket.ReplyCodeDataBanned.BanTypeData0(minutes_remaining=minutes_remaining)
                    result._byte_size = reader.position - reader_start_position
                    return result
                finally:
                    reader.chunked_reading_mode = old_chunked_reading_mode

            def __repr__(self):
                return f"InitInitServerPacket.ReplyCodeDataBanned.BanTypeData0(byte_size={repr(self._byte_size)}, minutes_remaining={repr(self._minutes_remaining)})"

        class BanTypeDataTemporary:
            """
            Data associated with ban_type value InitBanType.Temporary
            """
            _byte_size: int = 0
            _minutes_remaining: int

            def __init__(self, *, minutes_remaining: int):
                """
                Create a new instance of InitInitServerPacket.ReplyCodeDataBanned.BanTypeDataTemporary.

                Args:
                    minutes_remaining (int): (Value range is 0-255.)
                """
                self._minutes_remaining = minutes_remaining

            @property
            def byte_size(self) -> int:
                """
                Returns the size of the data that this was deserialized from.

                Returns:
                    int: The size of the data that this was deserialized from.
                """
                return self._byte_size

            @property
            def minutes_remaining(self) -> int:
                return self._minutes_remaining

            @staticmethod
            def serialize(writer: EoWriter, data: "InitInitServerPacket.ReplyCodeDataBanned.BanTypeDataTemporary") -> None:
                """
                Serializes an instance of `InitInitServerPacket.ReplyCodeDataBanned.BanTypeDataTemporary` to the provided `EoWriter`.

                Args:
                    writer (EoWriter): The writer that the data will be serialized to.
                    data (InitInitServerPacket.ReplyCodeDataBanned.BanTypeDataTemporary): The data to serialize.
                """
                old_string_sanitization_mode: bool = writer.string_sanitization_mode
                try:
                    if data._minutes_remaining is None:
                        raise SerializationError("minutes_remaining must be provided.")
                    writer.add_byte(data._minutes_remaining)
                finally:
                    writer.string_sanitization_mode = old_string_sanitization_mode

            @staticmethod
            def deserialize(reader: EoReader) -> "InitInitServerPacket.ReplyCodeDataBanned.BanTypeDataTemporary":
                """
                Deserializes an instance of `InitInitServerPacket.ReplyCodeDataBanned.BanTypeDataTemporary` from the provided `EoReader`.

                Args:
                    reader (EoReader): The writer that the data will be serialized to.

                Returns:
                    InitInitServerPacket.ReplyCodeDataBanned.BanTypeDataTemporary: The data to serialize.
                """
                old_chunked_reading_mode: bool = reader.chunked_reading_mode
                try:
                    reader_start_position: int = reader.position
                    minutes_remaining = reader.get_byte()
                    result = InitInitServerPacket.ReplyCodeDataBanned.BanTypeDataTemporary(minutes_remaining=minutes_remaining)
                    result._byte_size = reader.position - reader_start_position
                    return result
                finally:
                    reader.chunked_reading_mode = old_chunked_reading_mode

            def __repr__(self):
                return f"InitInitServerPacket.ReplyCodeDataBanned.BanTypeDataTemporary(byte_size={repr(self._byte_size)}, minutes_remaining={repr(self._minutes_remaining)})"

    class ReplyCodeDataWarpMap:
        """
        Data associated with reply_code value InitReply.WarpMap
        """
        _byte_size: int = 0
        _map_file: MapFile

        def __init__(self, *, map_file: MapFile):
            """
            Create a new instance of InitInitServerPacket.ReplyCodeDataWarpMap.

            Args:
                map_file (MapFile): 
            """
            self._map_file = map_file

        @property
        def byte_size(self) -> int:
            """
            Returns the size of the data that this was deserialized from.

            Returns:
                int: The size of the data that this was deserialized from.
            """
            return self._byte_size

        @property
        def map_file(self) -> MapFile:
            return self._map_file

        @staticmethod
        def serialize(writer: EoWriter, data: "InitInitServerPacket.ReplyCodeDataWarpMap") -> None:
            """
            Serializes an instance of `InitInitServerPacket.ReplyCodeDataWarpMap` to the provided `EoWriter`.

            Args:
                writer (EoWriter): The writer that the data will be serialized to.
                data (InitInitServerPacket.ReplyCodeDataWarpMap): The data to serialize.
            """
            old_string_sanitization_mode: bool = writer.string_sanitization_mode
            try:
                if data._map_file is None:
                    raise SerializationError("map_file must be provided.")
                MapFile.serialize(writer, data._map_file)
            finally:
                writer.string_sanitization_mode = old_string_sanitization_mode

        @staticmethod
        def deserialize(reader: EoReader) -> "InitInitServerPacket.ReplyCodeDataWarpMap":
            """
            Deserializes an instance of `InitInitServerPacket.ReplyCodeDataWarpMap` from the provided `EoReader`.

            Args:
                reader (EoReader): The writer that the data will be serialized to.

            Returns:
                InitInitServerPacket.ReplyCodeDataWarpMap: The data to serialize.
            """
            old_chunked_reading_mode: bool = reader.chunked_reading_mode
            try:
                reader_start_position: int = reader.position
                map_file = MapFile.deserialize(reader)
                result = InitInitServerPacket.ReplyCodeDataWarpMap(map_file=map_file)
                result._byte_size = reader.position - reader_start_position
                return result
            finally:
                reader.chunked_reading_mode = old_chunked_reading_mode

        def __repr__(self):
            return f"InitInitServerPacket.ReplyCodeDataWarpMap(byte_size={repr(self._byte_size)}, map_file={repr(self._map_file)})"

    class ReplyCodeDataFileEmf:
        """
        Data associated with reply_code value InitReply.FileEmf
        """
        _byte_size: int = 0
        _map_file: MapFile

        def __init__(self, *, map_file: MapFile):
            """
            Create a new instance of InitInitServerPacket.ReplyCodeDataFileEmf.

            Args:
                map_file (MapFile): 
            """
            self._map_file = map_file

        @property
        def byte_size(self) -> int:
            """
            Returns the size of the data that this was deserialized from.

            Returns:
                int: The size of the data that this was deserialized from.
            """
            return self._byte_size

        @property
        def map_file(self) -> MapFile:
            return self._map_file

        @staticmethod
        def serialize(writer: EoWriter, data: "InitInitServerPacket.ReplyCodeDataFileEmf") -> None:
            """
            Serializes an instance of `InitInitServerPacket.ReplyCodeDataFileEmf` to the provided `EoWriter`.

            Args:
                writer (EoWriter): The writer that the data will be serialized to.
                data (InitInitServerPacket.ReplyCodeDataFileEmf): The data to serialize.
            """
            old_string_sanitization_mode: bool = writer.string_sanitization_mode
            try:
                if data._map_file is None:
                    raise SerializationError("map_file must be provided.")
                MapFile.serialize(writer, data._map_file)
            finally:
                writer.string_sanitization_mode = old_string_sanitization_mode

        @staticmethod
        def deserialize(reader: EoReader) -> "InitInitServerPacket.ReplyCodeDataFileEmf":
            """
            Deserializes an instance of `InitInitServerPacket.ReplyCodeDataFileEmf` from the provided `EoReader`.

            Args:
                reader (EoReader): The writer that the data will be serialized to.

            Returns:
                InitInitServerPacket.ReplyCodeDataFileEmf: The data to serialize.
            """
            old_chunked_reading_mode: bool = reader.chunked_reading_mode
            try:
                reader_start_position: int = reader.position
                map_file = MapFile.deserialize(reader)
                result = InitInitServerPacket.ReplyCodeDataFileEmf(map_file=map_file)
                result._byte_size = reader.position - reader_start_position
                return result
            finally:
                reader.chunked_reading_mode = old_chunked_reading_mode

        def __repr__(self):
            return f"InitInitServerPacket.ReplyCodeDataFileEmf(byte_size={repr(self._byte_size)}, map_file={repr(self._map_file)})"

    class ReplyCodeDataFileEif:
        """
        Data associated with reply_code value InitReply.FileEif
        """
        _byte_size: int = 0
        _pub_file: PubFile

        def __init__(self, *, pub_file: PubFile):
            """
            Create a new instance of InitInitServerPacket.ReplyCodeDataFileEif.

            Args:
                pub_file (PubFile): 
            """
            self._pub_file = pub_file

        @property
        def byte_size(self) -> int:
            """
            Returns the size of the data that this was deserialized from.

            Returns:
                int: The size of the data that this was deserialized from.
            """
            return self._byte_size

        @property
        def pub_file(self) -> PubFile:
            return self._pub_file

        @staticmethod
        def serialize(writer: EoWriter, data: "InitInitServerPacket.ReplyCodeDataFileEif") -> None:
            """
            Serializes an instance of `InitInitServerPacket.ReplyCodeDataFileEif` to the provided `EoWriter`.

            Args:
                writer (EoWriter): The writer that the data will be serialized to.
                data (InitInitServerPacket.ReplyCodeDataFileEif): The data to serialize.
            """
            old_string_sanitization_mode: bool = writer.string_sanitization_mode
            try:
                if data._pub_file is None:
                    raise SerializationError("pub_file must be provided.")
                PubFile.serialize(writer, data._pub_file)
            finally:
                writer.string_sanitization_mode = old_string_sanitization_mode

        @staticmethod
        def deserialize(reader: EoReader) -> "InitInitServerPacket.ReplyCodeDataFileEif":
            """
            Deserializes an instance of `InitInitServerPacket.ReplyCodeDataFileEif` from the provided `EoReader`.

            Args:
                reader (EoReader): The writer that the data will be serialized to.

            Returns:
                InitInitServerPacket.ReplyCodeDataFileEif: The data to serialize.
            """
            old_chunked_reading_mode: bool = reader.chunked_reading_mode
            try:
                reader_start_position: int = reader.position
                pub_file = PubFile.deserialize(reader)
                result = InitInitServerPacket.ReplyCodeDataFileEif(pub_file=pub_file)
                result._byte_size = reader.position - reader_start_position
                return result
            finally:
                reader.chunked_reading_mode = old_chunked_reading_mode

        def __repr__(self):
            return f"InitInitServerPacket.ReplyCodeDataFileEif(byte_size={repr(self._byte_size)}, pub_file={repr(self._pub_file)})"

    class ReplyCodeDataFileEnf:
        """
        Data associated with reply_code value InitReply.FileEnf
        """
        _byte_size: int = 0
        _pub_file: PubFile

        def __init__(self, *, pub_file: PubFile):
            """
            Create a new instance of InitInitServerPacket.ReplyCodeDataFileEnf.

            Args:
                pub_file (PubFile): 
            """
            self._pub_file = pub_file

        @property
        def byte_size(self) -> int:
            """
            Returns the size of the data that this was deserialized from.

            Returns:
                int: The size of the data that this was deserialized from.
            """
            return self._byte_size

        @property
        def pub_file(self) -> PubFile:
            return self._pub_file

        @staticmethod
        def serialize(writer: EoWriter, data: "InitInitServerPacket.ReplyCodeDataFileEnf") -> None:
            """
            Serializes an instance of `InitInitServerPacket.ReplyCodeDataFileEnf` to the provided `EoWriter`.

            Args:
                writer (EoWriter): The writer that the data will be serialized to.
                data (InitInitServerPacket.ReplyCodeDataFileEnf): The data to serialize.
            """
            old_string_sanitization_mode: bool = writer.string_sanitization_mode
            try:
                if data._pub_file is None:
                    raise SerializationError("pub_file must be provided.")
                PubFile.serialize(writer, data._pub_file)
            finally:
                writer.string_sanitization_mode = old_string_sanitization_mode

        @staticmethod
        def deserialize(reader: EoReader) -> "InitInitServerPacket.ReplyCodeDataFileEnf":
            """
            Deserializes an instance of `InitInitServerPacket.ReplyCodeDataFileEnf` from the provided `EoReader`.

            Args:
                reader (EoReader): The writer that the data will be serialized to.

            Returns:
                InitInitServerPacket.ReplyCodeDataFileEnf: The data to serialize.
            """
            old_chunked_reading_mode: bool = reader.chunked_reading_mode
            try:
                reader_start_position: int = reader.position
                pub_file = PubFile.deserialize(reader)
                result = InitInitServerPacket.ReplyCodeDataFileEnf(pub_file=pub_file)
                result._byte_size = reader.position - reader_start_position
                return result
            finally:
                reader.chunked_reading_mode = old_chunked_reading_mode

        def __repr__(self):
            return f"InitInitServerPacket.ReplyCodeDataFileEnf(byte_size={repr(self._byte_size)}, pub_file={repr(self._pub_file)})"

    class ReplyCodeDataFileEsf:
        """
        Data associated with reply_code value InitReply.FileEsf
        """
        _byte_size: int = 0
        _pub_file: PubFile

        def __init__(self, *, pub_file: PubFile):
            """
            Create a new instance of InitInitServerPacket.ReplyCodeDataFileEsf.

            Args:
                pub_file (PubFile): 
            """
            self._pub_file = pub_file

        @property
        def byte_size(self) -> int:
            """
            Returns the size of the data that this was deserialized from.

            Returns:
                int: The size of the data that this was deserialized from.
            """
            return self._byte_size

        @property
        def pub_file(self) -> PubFile:
            return self._pub_file

        @staticmethod
        def serialize(writer: EoWriter, data: "InitInitServerPacket.ReplyCodeDataFileEsf") -> None:
            """
            Serializes an instance of `InitInitServerPacket.ReplyCodeDataFileEsf` to the provided `EoWriter`.

            Args:
                writer (EoWriter): The writer that the data will be serialized to.
                data (InitInitServerPacket.ReplyCodeDataFileEsf): The data to serialize.
            """
            old_string_sanitization_mode: bool = writer.string_sanitization_mode
            try:
                if data._pub_file is None:
                    raise SerializationError("pub_file must be provided.")
                PubFile.serialize(writer, data._pub_file)
            finally:
                writer.string_sanitization_mode = old_string_sanitization_mode

        @staticmethod
        def deserialize(reader: EoReader) -> "InitInitServerPacket.ReplyCodeDataFileEsf":
            """
            Deserializes an instance of `InitInitServerPacket.ReplyCodeDataFileEsf` from the provided `EoReader`.

            Args:
                reader (EoReader): The writer that the data will be serialized to.

            Returns:
                InitInitServerPacket.ReplyCodeDataFileEsf: The data to serialize.
            """
            old_chunked_reading_mode: bool = reader.chunked_reading_mode
            try:
                reader_start_position: int = reader.position
                pub_file = PubFile.deserialize(reader)
                result = InitInitServerPacket.ReplyCodeDataFileEsf(pub_file=pub_file)
                result._byte_size = reader.position - reader_start_position
                return result
            finally:
                reader.chunked_reading_mode = old_chunked_reading_mode

        def __repr__(self):
            return f"InitInitServerPacket.ReplyCodeDataFileEsf(byte_size={repr(self._byte_size)}, pub_file={repr(self._pub_file)})"

    class ReplyCodeDataFileEcf:
        """
        Data associated with reply_code value InitReply.FileEcf
        """
        _byte_size: int = 0
        _pub_file: PubFile

        def __init__(self, *, pub_file: PubFile):
            """
            Create a new instance of InitInitServerPacket.ReplyCodeDataFileEcf.

            Args:
                pub_file (PubFile): 
            """
            self._pub_file = pub_file

        @property
        def byte_size(self) -> int:
            """
            Returns the size of the data that this was deserialized from.

            Returns:
                int: The size of the data that this was deserialized from.
            """
            return self._byte_size

        @property
        def pub_file(self) -> PubFile:
            return self._pub_file

        @staticmethod
        def serialize(writer: EoWriter, data: "InitInitServerPacket.ReplyCodeDataFileEcf") -> None:
            """
            Serializes an instance of `InitInitServerPacket.ReplyCodeDataFileEcf` to the provided `EoWriter`.

            Args:
                writer (EoWriter): The writer that the data will be serialized to.
                data (InitInitServerPacket.ReplyCodeDataFileEcf): The data to serialize.
            """
            old_string_sanitization_mode: bool = writer.string_sanitization_mode
            try:
                if data._pub_file is None:
                    raise SerializationError("pub_file must be provided.")
                PubFile.serialize(writer, data._pub_file)
            finally:
                writer.string_sanitization_mode = old_string_sanitization_mode

        @staticmethod
        def deserialize(reader: EoReader) -> "InitInitServerPacket.ReplyCodeDataFileEcf":
            """
            Deserializes an instance of `InitInitServerPacket.ReplyCodeDataFileEcf` from the provided `EoReader`.

            Args:
                reader (EoReader): The writer that the data will be serialized to.

            Returns:
                InitInitServerPacket.ReplyCodeDataFileEcf: The data to serialize.
            """
            old_chunked_reading_mode: bool = reader.chunked_reading_mode
            try:
                reader_start_position: int = reader.position
                pub_file = PubFile.deserialize(reader)
                result = InitInitServerPacket.ReplyCodeDataFileEcf(pub_file=pub_file)
                result._byte_size = reader.position - reader_start_position
                return result
            finally:
                reader.chunked_reading_mode = old_chunked_reading_mode

        def __repr__(self):
            return f"InitInitServerPacket.ReplyCodeDataFileEcf(byte_size={repr(self._byte_size)}, pub_file={repr(self._pub_file)})"

    class ReplyCodeDataMapMutation:
        """
        Data associated with reply_code value InitReply.MapMutation
        """
        _byte_size: int = 0
        _map_file: MapFile

        def __init__(self, *, map_file: MapFile):
            """
            Create a new instance of InitInitServerPacket.ReplyCodeDataMapMutation.

            Args:
                map_file (MapFile): 
            """
            self._map_file = map_file

        @property
        def byte_size(self) -> int:
            """
            Returns the size of the data that this was deserialized from.

            Returns:
                int: The size of the data that this was deserialized from.
            """
            return self._byte_size

        @property
        def map_file(self) -> MapFile:
            return self._map_file

        @staticmethod
        def serialize(writer: EoWriter, data: "InitInitServerPacket.ReplyCodeDataMapMutation") -> None:
            """
            Serializes an instance of `InitInitServerPacket.ReplyCodeDataMapMutation` to the provided `EoWriter`.

            Args:
                writer (EoWriter): The writer that the data will be serialized to.
                data (InitInitServerPacket.ReplyCodeDataMapMutation): The data to serialize.
            """
            old_string_sanitization_mode: bool = writer.string_sanitization_mode
            try:
                if data._map_file is None:
                    raise SerializationError("map_file must be provided.")
                MapFile.serialize(writer, data._map_file)
            finally:
                writer.string_sanitization_mode = old_string_sanitization_mode

        @staticmethod
        def deserialize(reader: EoReader) -> "InitInitServerPacket.ReplyCodeDataMapMutation":
            """
            Deserializes an instance of `InitInitServerPacket.ReplyCodeDataMapMutation` from the provided `EoReader`.

            Args:
                reader (EoReader): The writer that the data will be serialized to.

            Returns:
                InitInitServerPacket.ReplyCodeDataMapMutation: The data to serialize.
            """
            old_chunked_reading_mode: bool = reader.chunked_reading_mode
            try:
                reader_start_position: int = reader.position
                map_file = MapFile.deserialize(reader)
                result = InitInitServerPacket.ReplyCodeDataMapMutation(map_file=map_file)
                result._byte_size = reader.position - reader_start_position
                return result
            finally:
                reader.chunked_reading_mode = old_chunked_reading_mode

        def __repr__(self):
            return f"InitInitServerPacket.ReplyCodeDataMapMutation(byte_size={repr(self._byte_size)}, map_file={repr(self._map_file)})"

    class ReplyCodeDataPlayersList:
        """
        Data associated with reply_code value InitReply.PlayersList
        """
        _byte_size: int = 0
        _players_list: PlayersList

        def __init__(self, *, players_list: PlayersList):
            """
            Create a new instance of InitInitServerPacket.ReplyCodeDataPlayersList.

            Args:
                players_list (PlayersList): 
            """
            self._players_list = players_list

        @property
        def byte_size(self) -> int:
            """
            Returns the size of the data that this was deserialized from.

            Returns:
                int: The size of the data that this was deserialized from.
            """
            return self._byte_size

        @property
        def players_list(self) -> PlayersList:
            return self._players_list

        @staticmethod
        def serialize(writer: EoWriter, data: "InitInitServerPacket.ReplyCodeDataPlayersList") -> None:
            """
            Serializes an instance of `InitInitServerPacket.ReplyCodeDataPlayersList` to the provided `EoWriter`.

            Args:
                writer (EoWriter): The writer that the data will be serialized to.
                data (InitInitServerPacket.ReplyCodeDataPlayersList): The data to serialize.
            """
            old_string_sanitization_mode: bool = writer.string_sanitization_mode
            try:
                writer.string_sanitization_mode = True
                if data._players_list is None:
                    raise SerializationError("players_list must be provided.")
                PlayersList.serialize(writer, data._players_list)
                writer.string_sanitization_mode = False
            finally:
                writer.string_sanitization_mode = old_string_sanitization_mode

        @staticmethod
        def deserialize(reader: EoReader) -> "InitInitServerPacket.ReplyCodeDataPlayersList":
            """
            Deserializes an instance of `InitInitServerPacket.ReplyCodeDataPlayersList` from the provided `EoReader`.

            Args:
                reader (EoReader): The writer that the data will be serialized to.

            Returns:
                InitInitServerPacket.ReplyCodeDataPlayersList: The data to serialize.
            """
            old_chunked_reading_mode: bool = reader.chunked_reading_mode
            try:
                reader_start_position: int = reader.position
                reader.chunked_reading_mode = True
                players_list = PlayersList.deserialize(reader)
                reader.chunked_reading_mode = False
                result = InitInitServerPacket.ReplyCodeDataPlayersList(players_list=players_list)
                result._byte_size = reader.position - reader_start_position
                return result
            finally:
                reader.chunked_reading_mode = old_chunked_reading_mode

        def __repr__(self):
            return f"InitInitServerPacket.ReplyCodeDataPlayersList(byte_size={repr(self._byte_size)}, players_list={repr(self._players_list)})"

    class ReplyCodeDataPlayersListFriends:
        """
        Data associated with reply_code value InitReply.PlayersListFriends
        """
        _byte_size: int = 0
        _players_list: PlayersListFriends

        def __init__(self, *, players_list: PlayersListFriends):
            """
            Create a new instance of InitInitServerPacket.ReplyCodeDataPlayersListFriends.

            Args:
                players_list (PlayersListFriends): 
            """
            self._players_list = players_list

        @property
        def byte_size(self) -> int:
            """
            Returns the size of the data that this was deserialized from.

            Returns:
                int: The size of the data that this was deserialized from.
            """
            return self._byte_size

        @property
        def players_list(self) -> PlayersListFriends:
            return self._players_list

        @staticmethod
        def serialize(writer: EoWriter, data: "InitInitServerPacket.ReplyCodeDataPlayersListFriends") -> None:
            """
            Serializes an instance of `InitInitServerPacket.ReplyCodeDataPlayersListFriends` to the provided `EoWriter`.

            Args:
                writer (EoWriter): The writer that the data will be serialized to.
                data (InitInitServerPacket.ReplyCodeDataPlayersListFriends): The data to serialize.
            """
            old_string_sanitization_mode: bool = writer.string_sanitization_mode
            try:
                writer.string_sanitization_mode = True
                if data._players_list is None:
                    raise SerializationError("players_list must be provided.")
                PlayersListFriends.serialize(writer, data._players_list)
                writer.string_sanitization_mode = False
            finally:
                writer.string_sanitization_mode = old_string_sanitization_mode

        @staticmethod
        def deserialize(reader: EoReader) -> "InitInitServerPacket.ReplyCodeDataPlayersListFriends":
            """
            Deserializes an instance of `InitInitServerPacket.ReplyCodeDataPlayersListFriends` from the provided `EoReader`.

            Args:
                reader (EoReader): The writer that the data will be serialized to.

            Returns:
                InitInitServerPacket.ReplyCodeDataPlayersListFriends: The data to serialize.
            """
            old_chunked_reading_mode: bool = reader.chunked_reading_mode
            try:
                reader_start_position: int = reader.position
                reader.chunked_reading_mode = True
                players_list = PlayersListFriends.deserialize(reader)
                reader.chunked_reading_mode = False
                result = InitInitServerPacket.ReplyCodeDataPlayersListFriends(players_list=players_list)
                result._byte_size = reader.position - reader_start_position
                return result
            finally:
                reader.chunked_reading_mode = old_chunked_reading_mode

        def __repr__(self):
            return f"InitInitServerPacket.ReplyCodeDataPlayersListFriends(byte_size={repr(self._byte_size)}, players_list={repr(self._players_list)})"



# Generated from the eo-protocol XML specification.
#
# This file should not be modified.
# Changes will be lost when code is regenerated.

from typing import Union
from .warp_type import WarpType
from .warp_effect import WarpEffect
from .nearby_info import NearbyInfo
from ..packet_family import PacketFamily
from ..packet_action import PacketAction
from ....serialization_error import SerializationError
from ....net.packet import Packet
from .....data.eo_writer import EoWriter
from .....data.eo_reader import EoReader

class WarpAgreeServerPacket(Packet):
    """
    Reply after accepting a warp
    """
    _byte_size: int = 0
    _warp_type: WarpType
    _warp_type_data: 'WarpAgreeServerPacket.WarpTypeData'
    _nearby: NearbyInfo

    def __init__(self, *, warp_type: WarpType, warp_type_data: 'WarpAgreeServerPacket.WarpTypeData' = None, nearby: NearbyInfo):
        """
        Create a new instance of WarpAgreeServerPacket.

        Args:
            warp_type (WarpType): 
            warp_type_data (WarpAgreeServerPacket.WarpTypeData): Data associated with the `warp_type` field.
            nearby (NearbyInfo): 
        """
        self._warp_type = warp_type
        self._warp_type_data = warp_type_data
        self._nearby = nearby

    @property
    def byte_size(self) -> int:
        """
        Returns the size of the data that this was deserialized from.

        Returns:
            int: The size of the data that this was deserialized from.
        """
        return self._byte_size

    @property
    def warp_type(self) -> WarpType:
        return self._warp_type

    @property
    def warp_type_data(self) -> 'WarpAgreeServerPacket.WarpTypeData':
        """
        WarpAgreeServerPacket.WarpTypeData: Data associated with the `warp_type` field.
        """
        return self._warp_type_data

    @property
    def nearby(self) -> NearbyInfo:
        return self._nearby

    @staticmethod
    def family() -> PacketFamily:
        """
        Returns the packet family associated with this packet.

        Returns:
            PacketFamily: The packet family associated with this packet.
        """
        return PacketFamily.Warp

    @staticmethod
    def action() -> PacketAction:
        """
        Returns the packet action associated with this packet.

        Returns:
            PacketAction: The packet action associated with this packet.
        """
        return PacketAction.Agree

    def write(self, writer):
        """
        Serializes and writes this packet to the provided EoWriter.

        Args:
            writer (EoWriter): the writer that this packet will be written to.
        """
        WarpAgreeServerPacket.serialize(writer, self)

    @staticmethod
    def serialize(writer: EoWriter, data: "WarpAgreeServerPacket") -> None:
        """
        Serializes an instance of `WarpAgreeServerPacket` to the provided `EoWriter`.

        Args:
            writer (EoWriter): The writer that the data will be serialized to.
            data (WarpAgreeServerPacket): The data to serialize.
        """
        old_string_sanitization_mode: bool = writer.string_sanitization_mode
        try:
            writer.string_sanitization_mode = True
            if data._warp_type is None:
                raise SerializationError("warp_type must be provided.")
            writer.add_char(int(data._warp_type))
            if data._warp_type == WarpType.MapSwitch:
                if not isinstance(data._warp_type_data, WarpAgreeServerPacket.WarpTypeDataMapSwitch):
                    raise SerializationError("Expected warp_type_data to be type WarpAgreeServerPacket.WarpTypeDataMapSwitch for warp_type " + WarpType(data._warp_type).name + ".")
                WarpAgreeServerPacket.WarpTypeDataMapSwitch.serialize(writer, data._warp_type_data)
            if data._nearby is None:
                raise SerializationError("nearby must be provided.")
            NearbyInfo.serialize(writer, data._nearby)
            writer.string_sanitization_mode = False
        finally:
            writer.string_sanitization_mode = old_string_sanitization_mode

    @staticmethod
    def deserialize(reader: EoReader) -> "WarpAgreeServerPacket":
        """
        Deserializes an instance of `WarpAgreeServerPacket` from the provided `EoReader`.

        Args:
            reader (EoReader): The writer that the data will be serialized to.

        Returns:
            WarpAgreeServerPacket: The data to serialize.
        """
        old_chunked_reading_mode: bool = reader.chunked_reading_mode
        try:
            reader_start_position: int = reader.position
            reader.chunked_reading_mode = True
            warp_type = WarpType(reader.get_char())
            warp_type_data: WarpAgreeServerPacket.WarpTypeData = None
            if warp_type == WarpType.MapSwitch:
                warp_type_data = WarpAgreeServerPacket.WarpTypeDataMapSwitch.deserialize(reader)
            nearby = NearbyInfo.deserialize(reader)
            reader.chunked_reading_mode = False
            result = WarpAgreeServerPacket(warp_type=warp_type, warp_type_data=warp_type_data, nearby=nearby)
            result._byte_size = reader.position - reader_start_position
            return result
        finally:
            reader.chunked_reading_mode = old_chunked_reading_mode

    def __repr__(self):
        return f"WarpAgreeServerPacket(byte_size={repr(self._byte_size)}, warp_type={repr(self._warp_type)}, warp_type_data={repr(self._warp_type_data)}, nearby={repr(self._nearby)})"

    WarpTypeData = Union['WarpAgreeServerPacket.WarpTypeDataMapSwitch', None]
    """
    Data associated with different values of the `warp_type` field.
    """

    class WarpTypeDataMapSwitch:
        """
        Data associated with warp_type value WarpType.MapSwitch
        """
        _byte_size: int = 0
        _map_id: int
        _warp_effect: WarpEffect

        def __init__(self, *, map_id: int, warp_effect: WarpEffect):
            """
            Create a new instance of WarpAgreeServerPacket.WarpTypeDataMapSwitch.

            Args:
                map_id (int): (Value range is 0-64008.)
                warp_effect (WarpEffect): 
            """
            self._map_id = map_id
            self._warp_effect = warp_effect

        @property
        def byte_size(self) -> int:
            """
            Returns the size of the data that this was deserialized from.

            Returns:
                int: The size of the data that this was deserialized from.
            """
            return self._byte_size

        @property
        def map_id(self) -> int:
            return self._map_id

        @property
        def warp_effect(self) -> WarpEffect:
            return self._warp_effect

        @staticmethod
        def serialize(writer: EoWriter, data: "WarpAgreeServerPacket.WarpTypeDataMapSwitch") -> None:
            """
            Serializes an instance of `WarpAgreeServerPacket.WarpTypeDataMapSwitch` to the provided `EoWriter`.

            Args:
                writer (EoWriter): The writer that the data will be serialized to.
                data (WarpAgreeServerPacket.WarpTypeDataMapSwitch): The data to serialize.
            """
            old_string_sanitization_mode: bool = writer.string_sanitization_mode
            try:
                if data._map_id is None:
                    raise SerializationError("map_id must be provided.")
                writer.add_short(data._map_id)
                if data._warp_effect is None:
                    raise SerializationError("warp_effect must be provided.")
                writer.add_char(int(data._warp_effect))
            finally:
                writer.string_sanitization_mode = old_string_sanitization_mode

        @staticmethod
        def deserialize(reader: EoReader) -> "WarpAgreeServerPacket.WarpTypeDataMapSwitch":
            """
            Deserializes an instance of `WarpAgreeServerPacket.WarpTypeDataMapSwitch` from the provided `EoReader`.

            Args:
                reader (EoReader): The writer that the data will be serialized to.

            Returns:
                WarpAgreeServerPacket.WarpTypeDataMapSwitch: The data to serialize.
            """
            old_chunked_reading_mode: bool = reader.chunked_reading_mode
            try:
                reader_start_position: int = reader.position
                map_id = reader.get_short()
                warp_effect = WarpEffect(reader.get_char())
                result = WarpAgreeServerPacket.WarpTypeDataMapSwitch(map_id=map_id, warp_effect=warp_effect)
                result._byte_size = reader.position - reader_start_position
                return result
            finally:
                reader.chunked_reading_mode = old_chunked_reading_mode

        def __repr__(self):
            return f"WarpAgreeServerPacket.WarpTypeDataMapSwitch(byte_size={repr(self._byte_size)}, map_id={repr(self._map_id)}, warp_effect={repr(self._warp_effect)})"



# Generated from the eo-protocol XML specification.
#
# This file should not be modified.
# Changes will be lost when code is regenerated.

from enum import IntEnum
from ....protocol_enum_meta import ProtocolEnumMeta

class QuestRequirementIcon(IntEnum, metaclass=ProtocolEnumMeta):
    """
    Icon displayed for each quest in the Quest Progress window
    """
    Item = 3
    Talk = 5
    Kill = 8
    Step = 10



# Generated from the eo-protocol XML specification.
#
# This file should not be modified.
# Changes will be lost when code is regenerated.

from ..coords import Coords
from ...serialization_error import SerializationError
from ....data.eo_writer import EoWriter
from ....data.eo_reader import EoReader

class MapSign:
    """
    Sign EMF entity
    """
    _byte_size: int = 0
    _coords: Coords
    _string_data_length: int
    _string_data: str
    _title_length: int

    def __init__(self, *, coords: Coords, string_data: str, title_length: int):
        """
        Create a new instance of MapSign.

        Args:
            coords (Coords): 
            string_data (str): (Length must be 64007 or less.)
            title_length (int): (Value range is 0-252.)
        """
        self._coords = coords
        self._string_data = string_data
        self._string_data_length = len(self._string_data)
        self._title_length = title_length

    @property
    def byte_size(self) -> int:
        """
        Returns the size of the data that this was deserialized from.

        Returns:
            int: The size of the data that this was deserialized from.
        """
        return self._byte_size

    @property
    def coords(self) -> Coords:
        return self._coords

    @property
    def string_data(self) -> str:
        return self._string_data

    @property
    def title_length(self) -> int:
        return self._title_length

    @staticmethod
    def serialize(writer: EoWriter, data: "MapSign") -> None:
        """
        Serializes an instance of `MapSign` to the provided `EoWriter`.

        Args:
            writer (EoWriter): The writer that the data will be serialized to.
            data (MapSign): The data to serialize.
        """
        old_string_sanitization_mode: bool = writer.string_sanitization_mode
        try:
            if data._coords is None:
                raise SerializationError("coords must be provided.")
            Coords.serialize(writer, data._coords)
            if data._string_data_length is None:
                raise SerializationError("string_data_length must be provided.")
            writer.add_short(data._string_data_length + 1)
            if data._string_data is None:
                raise SerializationError("string_data must be provided.")
            if len(data._string_data) > 64007:
                raise SerializationError(f"Expected length of string_data to be 64007 or less, got {len(data._string_data)}.")
            writer.add_fixed_encoded_string(data._string_data, data._string_data_length, False)
            if data._title_length is None:
                raise SerializationError("title_length must be provided.")
            writer.add_char(data._title_length)
        finally:
            writer.string_sanitization_mode = old_string_sanitization_mode

    @staticmethod
    def deserialize(reader: EoReader) -> "MapSign":
        """
        Deserializes an instance of `MapSign` from the provided `EoReader`.

        Args:
            reader (EoReader): The writer that the data will be serialized to.

        Returns:
            MapSign: The data to serialize.
        """
        old_chunked_reading_mode: bool = reader.chunked_reading_mode
        try:
            reader_start_position: int = reader.position
            coords = Coords.deserialize(reader)
            string_data_length = reader.get_short() - 1
            string_data = reader.get_fixed_encoded_string(string_data_length, False)
            title_length = reader.get_char()
            result = MapSign(coords=coords, string_data=string_data, title_length=title_length)
            result._byte_size = reader.position - reader_start_position
            return result
        finally:
            reader.chunked_reading_mode = old_chunked_reading_mode

    def __repr__(self):
        return f"MapSign(byte_size={repr(self._byte_size)}, coords={repr(self._coords)}, string_data={repr(self._string_data)}, title_length={repr(self._title_length)})"



# Generated from the eo-protocol XML specification.
#
# This file should not be modified.
# Changes will be lost when code is regenerated.

from typing import Union
from .character_stats_equipment_change import CharacterStatsEquipmentChange
from ..weight import Weight
from ..packet_family import PacketFamily
from ..packet_action import PacketAction
from ..item import Item
from ...pub.item_type import ItemType
from ....serialization_error import SerializationError
from ....net.packet import Packet
from .....data.eo_writer import EoWriter
from .....data.eo_reader import EoReader

class ItemReplyServerPacket(Packet):
    """
    Reply to using an item
    """
    _byte_size: int = 0
    _item_type: ItemType
    _used_item: Item
    _weight: Weight
    _item_type_data: 'ItemReplyServerPacket.ItemTypeData'

    def __init__(self, *, item_type: ItemType, used_item: Item, weight: Weight, item_type_data: 'ItemReplyServerPacket.ItemTypeData' = None):
        """
        Create a new instance of ItemReplyServerPacket.

        Args:
            item_type (ItemType): 
            used_item (Item): 
            weight (Weight): 
            item_type_data (ItemReplyServerPacket.ItemTypeData): Data associated with the `item_type` field.
        """
        self._item_type = item_type
        self._used_item = used_item
        self._weight = weight
        self._item_type_data = item_type_data

    @property
    def byte_size(self) -> int:
        """
        Returns the size of the data that this was deserialized from.

        Returns:
            int: The size of the data that this was deserialized from.
        """
        return self._byte_size

    @property
    def item_type(self) -> ItemType:
        return self._item_type

    @property
    def used_item(self) -> Item:
        return self._used_item

    @property
    def weight(self) -> Weight:
        return self._weight

    @property
    def item_type_data(self) -> 'ItemReplyServerPacket.ItemTypeData':
        """
        ItemReplyServerPacket.ItemTypeData: Data associated with the `item_type` field.
        """
        return self._item_type_data

    @staticmethod
    def family() -> PacketFamily:
        """
        Returns the packet family associated with this packet.

        Returns:
            PacketFamily: The packet family associated with this packet.
        """
        return PacketFamily.Item

    @staticmethod
    def action() -> PacketAction:
        """
        Returns the packet action associated with this packet.

        Returns:
            PacketAction: The packet action associated with this packet.
        """
        return PacketAction.Reply

    def write(self, writer):
        """
        Serializes and writes this packet to the provided EoWriter.

        Args:
            writer (EoWriter): the writer that this packet will be written to.
        """
        ItemReplyServerPacket.serialize(writer, self)

    @staticmethod
    def serialize(writer: EoWriter, data: "ItemReplyServerPacket") -> None:
        """
        Serializes an instance of `ItemReplyServerPacket` to the provided `EoWriter`.

        Args:
            writer (EoWriter): The writer that the data will be serialized to.
            data (ItemReplyServerPacket): The data to serialize.
        """
        old_string_sanitization_mode: bool = writer.string_sanitization_mode
        try:
            if data._item_type is None:
                raise SerializationError("item_type must be provided.")
            writer.add_char(int(data._item_type))
            if data._used_item is None:
                raise SerializationError("used_item must be provided.")
            Item.serialize(writer, data._used_item)
            if data._weight is None:
                raise SerializationError("weight must be provided.")
            Weight.serialize(writer, data._weight)
            if data._item_type == ItemType.Heal:
                if not isinstance(data._item_type_data, ItemReplyServerPacket.ItemTypeDataHeal):
                    raise SerializationError("Expected item_type_data to be type ItemReplyServerPacket.ItemTypeDataHeal for item_type " + ItemType(data._item_type).name + ".")
                ItemReplyServerPacket.ItemTypeDataHeal.serialize(writer, data._item_type_data)
            elif data._item_type == ItemType.HairDye:
                if not isinstance(data._item_type_data, ItemReplyServerPacket.ItemTypeDataHairDye):
                    raise SerializationError("Expected item_type_data to be type ItemReplyServerPacket.ItemTypeDataHairDye for item_type " + ItemType(data._item_type).name + ".")
                ItemReplyServerPacket.ItemTypeDataHairDye.serialize(writer, data._item_type_data)
            elif data._item_type == ItemType.EffectPotion:
                if not isinstance(data._item_type_data, ItemReplyServerPacket.ItemTypeDataEffectPotion):
                    raise SerializationError("Expected item_type_data to be type ItemReplyServerPacket.ItemTypeDataEffectPotion for item_type " + ItemType(data._item_type).name + ".")
                ItemReplyServerPacket.ItemTypeDataEffectPotion.serialize(writer, data._item_type_data)
            elif data._item_type == ItemType.CureCurse:
                if not isinstance(data._item_type_data, ItemReplyServerPacket.ItemTypeDataCureCurse):
                    raise SerializationError("Expected item_type_data to be type ItemReplyServerPacket.ItemTypeDataCureCurse for item_type " + ItemType(data._item_type).name + ".")
                ItemReplyServerPacket.ItemTypeDataCureCurse.serialize(writer, data._item_type_data)
            elif data._item_type == ItemType.ExpReward:
                if not isinstance(data._item_type_data, ItemReplyServerPacket.ItemTypeDataExpReward):
                    raise SerializationError("Expected item_type_data to be type ItemReplyServerPacket.ItemTypeDataExpReward for item_type " + ItemType(data._item_type).name + ".")
                ItemReplyServerPacket.ItemTypeDataExpReward.serialize(writer, data._item_type_data)
        finally:
            writer.string_sanitization_mode = old_string_sanitization_mode

    @staticmethod
    def deserialize(reader: EoReader) -> "ItemReplyServerPacket":
        """
        Deserializes an instance of `ItemReplyServerPacket` from the provided `EoReader`.

        Args:
            reader (EoReader): The writer that the data will be serialized to.

        Returns:
            ItemReplyServerPacket: The data to serialize.
        """
        old_chunked_reading_mode: bool = reader.chunked_reading_mode
        try:
            reader_start_position: int = reader.position
            item_type = ItemType(reader.get_char())
            used_item = Item.deserialize(reader)
            weight = Weight.deserialize(reader)
            item_type_data: ItemReplyServerPacket.ItemTypeData = None
            if item_type == ItemType.Heal:
                item_type_data = ItemReplyServerPacket.ItemTypeDataHeal.deserialize(reader)
            elif item_type == ItemType.HairDye:
                item_type_data = ItemReplyServerPacket.ItemTypeDataHairDye.deserialize(reader)
            elif item_type == ItemType.EffectPotion:
                item_type_data = ItemReplyServerPacket.ItemTypeDataEffectPotion.deserialize(reader)
            elif item_type == ItemType.CureCurse:
                item_type_data = ItemReplyServerPacket.ItemTypeDataCureCurse.deserialize(reader)
            elif item_type == ItemType.ExpReward:
                item_type_data = ItemReplyServerPacket.ItemTypeDataExpReward.deserialize(reader)
            result = ItemReplyServerPacket(item_type=item_type, used_item=used_item, weight=weight, item_type_data=item_type_data)
            result._byte_size = reader.position - reader_start_position
            return result
        finally:
            reader.chunked_reading_mode = old_chunked_reading_mode

    def __repr__(self):
        return f"ItemReplyServerPacket(byte_size={repr(self._byte_size)}, item_type={repr(self._item_type)}, used_item={repr(self._used_item)}, weight={repr(self._weight)}, item_type_data={repr(self._item_type_data)})"

    ItemTypeData = Union['ItemReplyServerPacket.ItemTypeDataHeal', 'ItemReplyServerPacket.ItemTypeDataHairDye', 'ItemReplyServerPacket.ItemTypeDataEffectPotion', 'ItemReplyServerPacket.ItemTypeDataCureCurse', 'ItemReplyServerPacket.ItemTypeDataExpReward', None]
    """
    Data associated with different values of the `item_type` field.
    """

    class ItemTypeDataHeal:
        """
        Data associated with item_type value ItemType.Heal
        """
        _byte_size: int = 0
        _hp_gain: int
        _hp: int
        _tp: int

        def __init__(self, *, hp_gain: int, hp: int, tp: int):
            """
            Create a new instance of ItemReplyServerPacket.ItemTypeDataHeal.

            Args:
                hp_gain (int): (Value range is 0-4097152080.)
                hp (int): (Value range is 0-64008.)
                tp (int): (Value range is 0-64008.)
            """
            self._hp_gain = hp_gain
            self._hp = hp
            self._tp = tp

        @property
        def byte_size(self) -> int:
            """
            Returns the size of the data that this was deserialized from.

            Returns:
                int: The size of the data that this was deserialized from.
            """
            return self._byte_size

        @property
        def hp_gain(self) -> int:
            return self._hp_gain

        @property
        def hp(self) -> int:
            return self._hp

        @property
        def tp(self) -> int:
            return self._tp

        @staticmethod
        def serialize(writer: EoWriter, data: "ItemReplyServerPacket.ItemTypeDataHeal") -> None:
            """
            Serializes an instance of `ItemReplyServerPacket.ItemTypeDataHeal` to the provided `EoWriter`.

            Args:
                writer (EoWriter): The writer that the data will be serialized to.
                data (ItemReplyServerPacket.ItemTypeDataHeal): The data to serialize.
            """
            old_string_sanitization_mode: bool = writer.string_sanitization_mode
            try:
                if data._hp_gain is None:
                    raise SerializationError("hp_gain must be provided.")
                writer.add_int(data._hp_gain)
                if data._hp is None:
                    raise SerializationError("hp must be provided.")
                writer.add_short(data._hp)
                if data._tp is None:
                    raise SerializationError("tp must be provided.")
                writer.add_short(data._tp)
            finally:
                writer.string_sanitization_mode = old_string_sanitization_mode

        @staticmethod
        def deserialize(reader: EoReader) -> "ItemReplyServerPacket.ItemTypeDataHeal":
            """
            Deserializes an instance of `ItemReplyServerPacket.ItemTypeDataHeal` from the provided `EoReader`.

            Args:
                reader (EoReader): The writer that the data will be serialized to.

            Returns:
                ItemReplyServerPacket.ItemTypeDataHeal: The data to serialize.
            """
            old_chunked_reading_mode: bool = reader.chunked_reading_mode
            try:
                reader_start_position: int = reader.position
                hp_gain = reader.get_int()
                hp = reader.get_short()
                tp = reader.get_short()
                result = ItemReplyServerPacket.ItemTypeDataHeal(hp_gain=hp_gain, hp=hp, tp=tp)
                result._byte_size = reader.position - reader_start_position
                return result
            finally:
                reader.chunked_reading_mode = old_chunked_reading_mode

        def __repr__(self):
            return f"ItemReplyServerPacket.ItemTypeDataHeal(byte_size={repr(self._byte_size)}, hp_gain={repr(self._hp_gain)}, hp={repr(self._hp)}, tp={repr(self._tp)})"

    class ItemTypeDataHairDye:
        """
        Data associated with item_type value ItemType.HairDye
        """
        _byte_size: int = 0
        _hair_color: int

        def __init__(self, *, hair_color: int):
            """
            Create a new instance of ItemReplyServerPacket.ItemTypeDataHairDye.

            Args:
                hair_color (int): (Value range is 0-252.)
            """
            self._hair_color = hair_color

        @property
        def byte_size(self) -> int:
            """
            Returns the size of the data that this was deserialized from.

            Returns:
                int: The size of the data that this was deserialized from.
            """
            return self._byte_size

        @property
        def hair_color(self) -> int:
            return self._hair_color

        @staticmethod
        def serialize(writer: EoWriter, data: "ItemReplyServerPacket.ItemTypeDataHairDye") -> None:
            """
            Serializes an instance of `ItemReplyServerPacket.ItemTypeDataHairDye` to the provided `EoWriter`.

            Args:
                writer (EoWriter): The writer that the data will be serialized to.
                data (ItemReplyServerPacket.ItemTypeDataHairDye): The data to serialize.
            """
            old_string_sanitization_mode: bool = writer.string_sanitization_mode
            try:
                if data._hair_color is None:
                    raise SerializationError("hair_color must be provided.")
                writer.add_char(data._hair_color)
            finally:
                writer.string_sanitization_mode = old_string_sanitization_mode

        @staticmethod
        def deserialize(reader: EoReader) -> "ItemReplyServerPacket.ItemTypeDataHairDye":
            """
            Deserializes an instance of `ItemReplyServerPacket.ItemTypeDataHairDye` from the provided `EoReader`.

            Args:
                reader (EoReader): The writer that the data will be serialized to.

            Returns:
                ItemReplyServerPacket.ItemTypeDataHairDye: The data to serialize.
            """
            old_chunked_reading_mode: bool = reader.chunked_reading_mode
            try:
                reader_start_position: int = reader.position
                hair_color = reader.get_char()
                result = ItemReplyServerPacket.ItemTypeDataHairDye(hair_color=hair_color)
                result._byte_size = reader.position - reader_start_position
                return result
            finally:
                reader.chunked_reading_mode = old_chunked_reading_mode

        def __repr__(self):
            return f"ItemReplyServerPacket.ItemTypeDataHairDye(byte_size={repr(self._byte_size)}, hair_color={repr(self._hair_color)})"

    class ItemTypeDataEffectPotion:
        """
        Data associated with item_type value ItemType.EffectPotion
        """
        _byte_size: int = 0
        _effect_id: int

        def __init__(self, *, effect_id: int):
            """
            Create a new instance of ItemReplyServerPacket.ItemTypeDataEffectPotion.

            Args:
                effect_id (int): (Value range is 0-64008.)
            """
            self._effect_id = effect_id

        @property
        def byte_size(self) -> int:
            """
            Returns the size of the data that this was deserialized from.

            Returns:
                int: The size of the data that this was deserialized from.
            """
            return self._byte_size

        @property
        def effect_id(self) -> int:
            return self._effect_id

        @staticmethod
        def serialize(writer: EoWriter, data: "ItemReplyServerPacket.ItemTypeDataEffectPotion") -> None:
            """
            Serializes an instance of `ItemReplyServerPacket.ItemTypeDataEffectPotion` to the provided `EoWriter`.

            Args:
                writer (EoWriter): The writer that the data will be serialized to.
                data (ItemReplyServerPacket.ItemTypeDataEffectPotion): The data to serialize.
            """
            old_string_sanitization_mode: bool = writer.string_sanitization_mode
            try:
                if data._effect_id is None:
                    raise SerializationError("effect_id must be provided.")
                writer.add_short(data._effect_id)
            finally:
                writer.string_sanitization_mode = old_string_sanitization_mode

        @staticmethod
        def deserialize(reader: EoReader) -> "ItemReplyServerPacket.ItemTypeDataEffectPotion":
            """
            Deserializes an instance of `ItemReplyServerPacket.ItemTypeDataEffectPotion` from the provided `EoReader`.

            Args:
                reader (EoReader): The writer that the data will be serialized to.

            Returns:
                ItemReplyServerPacket.ItemTypeDataEffectPotion: The data to serialize.
            """
            old_chunked_reading_mode: bool = reader.chunked_reading_mode
            try:
                reader_start_position: int = reader.position
                effect_id = reader.get_short()
                result = ItemReplyServerPacket.ItemTypeDataEffectPotion(effect_id=effect_id)
                result._byte_size = reader.position - reader_start_position
                return result
            finally:
                reader.chunked_reading_mode = old_chunked_reading_mode

        def __repr__(self):
            return f"ItemReplyServerPacket.ItemTypeDataEffectPotion(byte_size={repr(self._byte_size)}, effect_id={repr(self._effect_id)})"

    class ItemTypeDataCureCurse:
        """
        Data associated with item_type value ItemType.CureCurse
        """
        _byte_size: int = 0
        _stats: CharacterStatsEquipmentChange

        def __init__(self, *, stats: CharacterStatsEquipmentChange):
            """
            Create a new instance of ItemReplyServerPacket.ItemTypeDataCureCurse.

            Args:
                stats (CharacterStatsEquipmentChange): 
            """
            self._stats = stats

        @property
        def byte_size(self) -> int:
            """
            Returns the size of the data that this was deserialized from.

            Returns:
                int: The size of the data that this was deserialized from.
            """
            return self._byte_size

        @property
        def stats(self) -> CharacterStatsEquipmentChange:
            return self._stats

        @staticmethod
        def serialize(writer: EoWriter, data: "ItemReplyServerPacket.ItemTypeDataCureCurse") -> None:
            """
            Serializes an instance of `ItemReplyServerPacket.ItemTypeDataCureCurse` to the provided `EoWriter`.

            Args:
                writer (EoWriter): The writer that the data will be serialized to.
                data (ItemReplyServerPacket.ItemTypeDataCureCurse): The data to serialize.
            """
            old_string_sanitization_mode: bool = writer.string_sanitization_mode
            try:
                if data._stats is None:
                    raise SerializationError("stats must be provided.")
                CharacterStatsEquipmentChange.serialize(writer, data._stats)
            finally:
                writer.string_sanitization_mode = old_string_sanitization_mode

        @staticmethod
        def deserialize(reader: EoReader) -> "ItemReplyServerPacket.ItemTypeDataCureCurse":
            """
            Deserializes an instance of `ItemReplyServerPacket.ItemTypeDataCureCurse` from the provided `EoReader`.

            Args:
                reader (EoReader): The writer that the data will be serialized to.

            Returns:
                ItemReplyServerPacket.ItemTypeDataCureCurse: The data to serialize.
            """
            old_chunked_reading_mode: bool = reader.chunked_reading_mode
            try:
                reader_start_position: int = reader.position
                stats = CharacterStatsEquipmentChange.deserialize(reader)
                result = ItemReplyServerPacket.ItemTypeDataCureCurse(stats=stats)
                result._byte_size = reader.position - reader_start_position
                return result
            finally:
                reader.chunked_reading_mode = old_chunked_reading_mode

        def __repr__(self):
            return f"ItemReplyServerPacket.ItemTypeDataCureCurse(byte_size={repr(self._byte_size)}, stats={repr(self._stats)})"

    class ItemTypeDataExpReward:
        """
        Data associated with item_type value ItemType.ExpReward
        """
        _byte_size: int = 0
        _experience: int
        _level_up: int
        _stat_points: int
        _skill_points: int
        _max_hp: int
        _max_tp: int
        _max_sp: int

        def __init__(self, *, experience: int, level_up: int, stat_points: int, skill_points: int, max_hp: int, max_tp: int, max_sp: int):
            """
            Create a new instance of ItemReplyServerPacket.ItemTypeDataExpReward.

            Args:
                experience (int): (Value range is 0-4097152080.)
                level_up (int): A value greater than 0 is "new level" and indicates the player leveled up. (Value range is 0-252.)
                stat_points (int): (Value range is 0-64008.)
                skill_points (int): (Value range is 0-64008.)
                max_hp (int): (Value range is 0-64008.)
                max_tp (int): (Value range is 0-64008.)
                max_sp (int): (Value range is 0-64008.)
            """
            self._experience = experience
            self._level_up = level_up
            self._stat_points = stat_points
            self._skill_points = skill_points
            self._max_hp = max_hp
            self._max_tp = max_tp
            self._max_sp = max_sp

        @property
        def byte_size(self) -> int:
            """
            Returns the size of the data that this was deserialized from.

            Returns:
                int: The size of the data that this was deserialized from.
            """
            return self._byte_size

        @property
        def experience(self) -> int:
            return self._experience

        @property
        def level_up(self) -> int:
            """
            A value greater than 0 is "new level" and indicates the player leveled up.
            """
            return self._level_up

        @property
        def stat_points(self) -> int:
            return self._stat_points

        @property
        def skill_points(self) -> int:
            return self._skill_points

        @property
        def max_hp(self) -> int:
            return self._max_hp

        @property
        def max_tp(self) -> int:
            return self._max_tp

        @property
        def max_sp(self) -> int:
            return self._max_sp

        @staticmethod
        def serialize(writer: EoWriter, data: "ItemReplyServerPacket.ItemTypeDataExpReward") -> None:
            """
            Serializes an instance of `ItemReplyServerPacket.ItemTypeDataExpReward` to the provided `EoWriter`.

            Args:
                writer (EoWriter): The writer that the data will be serialized to.
                data (ItemReplyServerPacket.ItemTypeDataExpReward): The data to serialize.
            """
            old_string_sanitization_mode: bool = writer.string_sanitization_mode
            try:
                if data._experience is None:
                    raise SerializationError("experience must be provided.")
                writer.add_int(data._experience)
                if data._level_up is None:
                    raise SerializationError("level_up must be provided.")
                writer.add_char(data._level_up)
                if data._stat_points is None:
                    raise SerializationError("stat_points must be provided.")
                writer.add_short(data._stat_points)
                if data._skill_points is None:
                    raise SerializationError("skill_points must be provided.")
                writer.add_short(data._skill_points)
                if data._max_hp is None:
                    raise SerializationError("max_hp must be provided.")
                writer.add_short(data._max_hp)
                if data._max_tp is None:
                    raise SerializationError("max_tp must be provided.")
                writer.add_short(data._max_tp)
                if data._max_sp is None:
                    raise SerializationError("max_sp must be provided.")
                writer.add_short(data._max_sp)
            finally:
                writer.string_sanitization_mode = old_string_sanitization_mode

        @staticmethod
        def deserialize(reader: EoReader) -> "ItemReplyServerPacket.ItemTypeDataExpReward":
            """
            Deserializes an instance of `ItemReplyServerPacket.ItemTypeDataExpReward` from the provided `EoReader`.

            Args:
                reader (EoReader): The writer that the data will be serialized to.

            Returns:
                ItemReplyServerPacket.ItemTypeDataExpReward: The data to serialize.
            """
            old_chunked_reading_mode: bool = reader.chunked_reading_mode
            try:
                reader_start_position: int = reader.position
                experience = reader.get_int()
                level_up = reader.get_char()
                stat_points = reader.get_short()
                skill_points = reader.get_short()
                max_hp = reader.get_short()
                max_tp = reader.get_short()
                max_sp = reader.get_short()
                result = ItemReplyServerPacket.ItemTypeDataExpReward(experience=experience, level_up=level_up, stat_points=stat_points, skill_points=skill_points, max_hp=max_hp, max_tp=max_tp, max_sp=max_sp)
                result._byte_size = reader.position - reader_start_position
                return result
            finally:
                reader.chunked_reading_mode = old_chunked_reading_mode

        def __repr__(self):
            return f"ItemReplyServerPacket.ItemTypeDataExpReward(byte_size={repr(self._byte_size)}, experience={repr(self._experience)}, level_up={repr(self._level_up)}, stat_points={repr(self._stat_points)}, skill_points={repr(self._skill_points)}, max_hp={repr(self._max_hp)}, max_tp={repr(self._max_tp)}, max_sp={repr(self._max_sp)})"



# Generated from the eo-protocol XML specification.
#
# This file should not be modified.
# Changes will be lost when code is regenerated.

from __future__ import annotations
from collections.abc import Iterable
from .skill_stat_requirements import SkillStatRequirements
from ....serialization_error import SerializationError
from .....data.eo_writer import EoWriter
from .....data.eo_reader import EoReader

class SkillLearn:
    """
    A skill that can be learned from a skill master NPC
    """
    _byte_size: int = 0
    _id: int
    _level_requirement: int
    _class_requirement: int
    _cost: int
    _skill_requirements: tuple[int, ...]
    _stat_requirements: SkillStatRequirements

    def __init__(self, *, id: int, level_requirement: int, class_requirement: int, cost: int, skill_requirements: Iterable[int], stat_requirements: SkillStatRequirements):
        """
        Create a new instance of SkillLearn.

        Args:
            id (int): (Value range is 0-64008.)
            level_requirement (int): (Value range is 0-252.)
            class_requirement (int): (Value range is 0-252.)
            cost (int): (Value range is 0-4097152080.)
            skill_requirements (Iterable[int]): (Length must be `4`.) (Element value range is 0-64008.)
            stat_requirements (SkillStatRequirements): 
        """
        self._id = id
        self._level_requirement = level_requirement
        self._class_requirement = class_requirement
        self._cost = cost
        self._skill_requirements = tuple(skill_requirements)
        self._stat_requirements = stat_requirements

    @property
    def byte_size(self) -> int:
        """
        Returns the size of the data that this was deserialized from.

        Returns:
            int: The size of the data that this was deserialized from.
        """
        return self._byte_size

    @property
    def id(self) -> int:
        return self._id

    @property
    def level_requirement(self) -> int:
        return self._level_requirement

    @property
    def class_requirement(self) -> int:
        return self._class_requirement

    @property
    def cost(self) -> int:
        return self._cost

    @property
    def skill_requirements(self) -> tuple[int, ...]:
        return self._skill_requirements

    @property
    def stat_requirements(self) -> SkillStatRequirements:
        return self._stat_requirements

    @staticmethod
    def serialize(writer: EoWriter, data: "SkillLearn") -> None:
        """
        Serializes an instance of `SkillLearn` to the provided `EoWriter`.

        Args:
            writer (EoWriter): The writer that the data will be serialized to.
            data (SkillLearn): The data to serialize.
        """
        old_string_sanitization_mode: bool = writer.string_sanitization_mode
        try:
            if data._id is None:
                raise SerializationError("id must be provided.")
            writer.add_short(data._id)
            if data._level_requirement is None:
                raise SerializationError("level_requirement must be provided.")
            writer.add_char(data._level_requirement)
            if data._class_requirement is None:
                raise SerializationError("class_requirement must be provided.")
            writer.add_char(data._class_requirement)
            if data._cost is None:
                raise SerializationError("cost must be provided.")
            writer.add_int(data._cost)
            if data._skill_requirements is None:
                raise SerializationError("skill_requirements must be provided.")
            if len(data._skill_requirements) != 4:
                raise SerializationError(f"Expected length of skill_requirements to be exactly 4, got {len(data._skill_requirements)}.")
            for i in range(4):
                writer.add_short(data._skill_requirements[i])
            if data._stat_requirements is None:
                raise SerializationError("stat_requirements must be provided.")
            SkillStatRequirements.serialize(writer, data._stat_requirements)
        finally:
            writer.string_sanitization_mode = old_string_sanitization_mode

    @staticmethod
    def deserialize(reader: EoReader) -> "SkillLearn":
        """
        Deserializes an instance of `SkillLearn` from the provided `EoReader`.

        Args:
            reader (EoReader): The writer that the data will be serialized to.

        Returns:
            SkillLearn: The data to serialize.
        """
        old_chunked_reading_mode: bool = reader.chunked_reading_mode
        try:
            reader_start_position: int = reader.position
            id = reader.get_short()
            level_requirement = reader.get_char()
            class_requirement = reader.get_char()
            cost = reader.get_int()
            skill_requirements = []
            for i in range(4):
                skill_requirements.append(reader.get_short())
            stat_requirements = SkillStatRequirements.deserialize(reader)
            result = SkillLearn(id=id, level_requirement=level_requirement, class_requirement=class_requirement, cost=cost, skill_requirements=skill_requirements, stat_requirements=stat_requirements)
            result._byte_size = reader.position - reader_start_position
            return result
        finally:
            reader.chunked_reading_mode = old_chunked_reading_mode

    def __repr__(self):
        return f"SkillLearn(byte_size={repr(self._byte_size)}, id={repr(self._id)}, level_requirement={repr(self._level_requirement)}, class_requirement={repr(self._class_requirement)}, cost={repr(self._cost)}, skill_requirements={repr(self._skill_requirements)}, stat_requirements={repr(self._stat_requirements)})"



# Generated from the eo-protocol XML specification.
#
# This file should not be modified.
# Changes will be lost when code is regenerated.

from .npc_type import NpcType
from .element import Element
from ...serialization_error import SerializationError
from ....data.eo_writer import EoWriter
from ....data.eo_reader import EoReader

class EnfRecord:
    """
    Record of NPC data in an Endless NPC File
    """
    _byte_size: int = 0
    _name_length: int
    _name: str
    _graphic_id: int
    _race: int
    _boss: bool
    _child: bool
    _type: NpcType
    _behavior_id: int
    _hp: int
    _tp: int
    _min_damage: int
    _max_damage: int
    _accuracy: int
    _evade: int
    _armor: int
    _return_damage: int
    _element: Element
    _element_damage: int
    _element_weakness: Element
    _element_weakness_damage: int
    _level: int
    _experience: int

    def __init__(self, *, name: str, graphic_id: int, race: int, boss: bool, child: bool, type: NpcType, behavior_id: int, hp: int, tp: int, min_damage: int, max_damage: int, accuracy: int, evade: int, armor: int, return_damage: int, element: Element, element_damage: int, element_weakness: Element, element_weakness_damage: int, level: int, experience: int):
        """
        Create a new instance of EnfRecord.

        Args:
            name (str): (Length must be 252 or less.)
            graphic_id (int): (Value range is 0-64008.)
            race (int): (Value range is 0-252.)
            boss (bool): 
            child (bool): 
            type (NpcType): 
            behavior_id (int): (Value range is 0-64008.)
            hp (int): (Value range is 0-16194276.)
            tp (int): (Value range is 0-64008.)
            min_damage (int): (Value range is 0-64008.)
            max_damage (int): (Value range is 0-64008.)
            accuracy (int): (Value range is 0-64008.)
            evade (int): (Value range is 0-64008.)
            armor (int): (Value range is 0-64008.)
            return_damage (int): (Value range is 0-252.)
            element (Element): 
            element_damage (int): (Value range is 0-64008.)
            element_weakness (Element): 
            element_weakness_damage (int): (Value range is 0-64008.)
            level (int): (Value range is 0-252.)
            experience (int): (Value range is 0-16194276.)
        """
        self._name = name
        self._name_length = len(self._name)
        self._graphic_id = graphic_id
        self._race = race
        self._boss = boss
        self._child = child
        self._type = type
        self._behavior_id = behavior_id
        self._hp = hp
        self._tp = tp
        self._min_damage = min_damage
        self._max_damage = max_damage
        self._accuracy = accuracy
        self._evade = evade
        self._armor = armor
        self._return_damage = return_damage
        self._element = element
        self._element_damage = element_damage
        self._element_weakness = element_weakness
        self._element_weakness_damage = element_weakness_damage
        self._level = level
        self._experience = experience

    @property
    def byte_size(self) -> int:
        """
        Returns the size of the data that this was deserialized from.

        Returns:
            int: The size of the data that this was deserialized from.
        """
        return self._byte_size

    @property
    def name(self) -> str:
        return self._name

    @property
    def graphic_id(self) -> int:
        return self._graphic_id

    @property
    def race(self) -> int:
        return self._race

    @property
    def boss(self) -> bool:
        return self._boss

    @property
    def child(self) -> bool:
        return self._child

    @property
    def type(self) -> NpcType:
        return self._type

    @property
    def behavior_id(self) -> int:
        return self._behavior_id

    @property
    def hp(self) -> int:
        return self._hp

    @property
    def tp(self) -> int:
        return self._tp

    @property
    def min_damage(self) -> int:
        return self._min_damage

    @property
    def max_damage(self) -> int:
        return self._max_damage

    @property
    def accuracy(self) -> int:
        return self._accuracy

    @property
    def evade(self) -> int:
        return self._evade

    @property
    def armor(self) -> int:
        return self._armor

    @property
    def return_damage(self) -> int:
        return self._return_damage

    @property
    def element(self) -> Element:
        return self._element

    @property
    def element_damage(self) -> int:
        return self._element_damage

    @property
    def element_weakness(self) -> Element:
        return self._element_weakness

    @property
    def element_weakness_damage(self) -> int:
        return self._element_weakness_damage

    @property
    def level(self) -> int:
        return self._level

    @property
    def experience(self) -> int:
        return self._experience

    @staticmethod
    def serialize(writer: EoWriter, data: "EnfRecord") -> None:
        """
        Serializes an instance of `EnfRecord` to the provided `EoWriter`.

        Args:
            writer (EoWriter): The writer that the data will be serialized to.
            data (EnfRecord): The data to serialize.
        """
        old_string_sanitization_mode: bool = writer.string_sanitization_mode
        try:
            if data._name_length is None:
                raise SerializationError("name_length must be provided.")
            writer.add_char(data._name_length)
            if data._name is None:
                raise SerializationError("name must be provided.")
            if len(data._name) > 252:
                raise SerializationError(f"Expected length of name to be 252 or less, got {len(data._name)}.")
            writer.add_fixed_string(data._name, data._name_length, False)
            if data._graphic_id is None:
                raise SerializationError("graphic_id must be provided.")
            writer.add_short(data._graphic_id)
            if data._race is None:
                raise SerializationError("race must be provided.")
            writer.add_char(data._race)
            if data._boss is None:
                raise SerializationError("boss must be provided.")
            writer.add_short(1 if data._boss else 0)
            if data._child is None:
                raise SerializationError("child must be provided.")
            writer.add_short(1 if data._child else 0)
            if data._type is None:
                raise SerializationError("type must be provided.")
            writer.add_short(int(data._type))
            if data._behavior_id is None:
                raise SerializationError("behavior_id must be provided.")
            writer.add_short(data._behavior_id)
            if data._hp is None:
                raise SerializationError("hp must be provided.")
            writer.add_three(data._hp)
            if data._tp is None:
                raise SerializationError("tp must be provided.")
            writer.add_short(data._tp)
            if data._min_damage is None:
                raise SerializationError("min_damage must be provided.")
            writer.add_short(data._min_damage)
            if data._max_damage is None:
                raise SerializationError("max_damage must be provided.")
            writer.add_short(data._max_damage)
            if data._accuracy is None:
                raise SerializationError("accuracy must be provided.")
            writer.add_short(data._accuracy)
            if data._evade is None:
                raise SerializationError("evade must be provided.")
            writer.add_short(data._evade)
            if data._armor is None:
                raise SerializationError("armor must be provided.")
            writer.add_short(data._armor)
            if data._return_damage is None:
                raise SerializationError("return_damage must be provided.")
            writer.add_char(data._return_damage)
            if data._element is None:
                raise SerializationError("element must be provided.")
            writer.add_short(int(data._element))
            if data._element_damage is None:
                raise SerializationError("element_damage must be provided.")
            writer.add_short(data._element_damage)
            if data._element_weakness is None:
                raise SerializationError("element_weakness must be provided.")
            writer.add_short(int(data._element_weakness))
            if data._element_weakness_damage is None:
                raise SerializationError("element_weakness_damage must be provided.")
            writer.add_short(data._element_weakness_damage)
            if data._level is None:
                raise SerializationError("level must be provided.")
            writer.add_char(data._level)
            if data._experience is None:
                raise SerializationError("experience must be provided.")
            writer.add_three(data._experience)
        finally:
            writer.string_sanitization_mode = old_string_sanitization_mode

    @staticmethod
    def deserialize(reader: EoReader) -> "EnfRecord":
        """
        Deserializes an instance of `EnfRecord` from the provided `EoReader`.

        Args:
            reader (EoReader): The writer that the data will be serialized to.

        Returns:
            EnfRecord: The data to serialize.
        """
        old_chunked_reading_mode: bool = reader.chunked_reading_mode
        try:
            reader_start_position: int = reader.position
            name_length = reader.get_char()
            name = reader.get_fixed_string(name_length, False)
            graphic_id = reader.get_short()
            race = reader.get_char()
            boss = reader.get_short() != 0
            child = reader.get_short() != 0
            type = NpcType(reader.get_short())
            behavior_id = reader.get_short()
            hp = reader.get_three()
            tp = reader.get_short()
            min_damage = reader.get_short()
            max_damage = reader.get_short()
            accuracy = reader.get_short()
            evade = reader.get_short()
            armor = reader.get_short()
            return_damage = reader.get_char()
            element = Element(reader.get_short())
            element_damage = reader.get_short()
            element_weakness = Element(reader.get_short())
            element_weakness_damage = reader.get_short()
            level = reader.get_char()
            experience = reader.get_three()
            result = EnfRecord(name=name, graphic_id=graphic_id, race=race, boss=boss, child=child, type=type, behavior_id=behavior_id, hp=hp, tp=tp, min_damage=min_damage, max_damage=max_damage, accuracy=accuracy, evade=evade, armor=armor, return_damage=return_damage, element=element, element_damage=element_damage, element_weakness=element_weakness, element_weakness_damage=element_weakness_damage, level=level, experience=experience)
            result._byte_size = reader.position - reader_start_position
            return result
        finally:
            reader.chunked_reading_mode = old_chunked_reading_mode

    def __repr__(self):
        return f"EnfRecord(byte_size={repr(self._byte_size)}, name={repr(self._name)}, graphic_id={repr(self._graphic_id)}, race={repr(self._race)}, boss={repr(self._boss)}, child={repr(self._child)}, type={repr(self._type)}, behavior_id={repr(self._behavior_id)}, hp={repr(self._hp)}, tp={repr(self._tp)}, min_damage={repr(self._min_damage)}, max_damage={repr(self._max_damage)}, accuracy={repr(self._accuracy)}, evade={repr(self._evade)}, armor={repr(self._armor)}, return_damage={repr(self._return_damage)}, element={repr(self._element)}, element_damage={repr(self._element_damage)}, element_weakness={repr(self._element_weakness)}, element_weakness_damage={repr(self._element_weakness_damage)}, level={repr(self._level)}, experience={repr(self._experience)})"



# Generated from the eo-protocol XML specification.
#
# This file should not be modified.
# Changes will be lost when code is regenerated.

"""
Data structures generated from the [eo-protocol](https://github.com/cirras/eo-protocol/tree/master/xml/net/client/protocol.xml){target="_blank"} XML specification.

Warning:
  - This subpackage should not be directly imported. 
  - Instead, import [eolib.protocol.net.client][] (or the top-level `eolib`).
"""

from .welcome_request_client_packet import *
from .welcome_msg_client_packet import *
from .welcome_agree_client_packet import *
from .warp_take_client_packet import *
from .warp_accept_client_packet import *
from .walk_spec_client_packet import *
from .walk_player_client_packet import *
from .walk_admin_client_packet import *
from .walk_action import *
from .train_type import *
from .trade_request_client_packet import *
from .trade_remove_client_packet import *
from .trade_close_client_packet import *
from .trade_agree_client_packet import *
from .trade_add_client_packet import *
from .trade_accept_client_packet import *
from .talk_use_client_packet import *
from .talk_tell_client_packet import *
from .talk_request_client_packet import *
from .talk_report_client_packet import *
from .talk_player_client_packet import *
from .talk_open_client_packet import *
from .talk_msg_client_packet import *
from .talk_announce_client_packet import *
from .talk_admin_client_packet import *
from .stat_skill_take_client_packet import *
from .stat_skill_remove_client_packet import *
from .stat_skill_open_client_packet import *
from .stat_skill_junk_client_packet import *
from .stat_skill_add_client_packet import *
from .stat_id import *
from .spell_use_client_packet import *
from .spell_target_type import *
from .spell_target_self_client_packet import *
from .spell_target_other_client_packet import *
from .spell_target_group_client_packet import *
from .spell_request_client_packet import *
from .sit_request_client_packet import *
from .sit_action import *
from .shop_sell_client_packet import *
from .shop_open_client_packet import *
from .shop_create_client_packet import *
from .shop_buy_client_packet import *
from .refresh_request_client_packet import *
from .range_request_client_packet import *
from .quest_use_client_packet import *
from .quest_list_client_packet import *
from .quest_accept_client_packet import *
from .priest_use_client_packet import *
from .priest_request_client_packet import *
from .priest_open_client_packet import *
from .priest_accept_client_packet import *
from .players_request_client_packet import *
from .players_list_client_packet import *
from .players_accept_client_packet import *
from .player_range_request_client_packet import *
from .party_take_client_packet import *
from .party_request_client_packet import *
from .party_remove_client_packet import *
from .party_accept_client_packet import *
from .paperdoll_request_client_packet import *
from .paperdoll_remove_client_packet import *
from .paperdoll_add_client_packet import *
from .npc_range_request_client_packet import *
from .message_ping_client_packet import *
from .marriage_request_type import *
from .marriage_request_client_packet import *
from .marriage_open_client_packet import *
from .login_request_client_packet import *
from .locker_take_client_packet import *
from .locker_open_client_packet import *
from .locker_buy_client_packet import *
from .locker_add_client_packet import *
from .jukebox_use_client_packet import *
from .jukebox_open_client_packet import *
from .jukebox_msg_client_packet import *
from .item_use_client_packet import *
from .item_junk_client_packet import *
from .item_get_client_packet import *
from .item_drop_client_packet import *
from .init_init_client_packet import *
from .guild_use_client_packet import *
from .guild_tell_client_packet import *
from .guild_take_client_packet import *
from .guild_request_client_packet import *
from .guild_report_client_packet import *
from .guild_remove_client_packet import *
from .guild_rank_client_packet import *
from .guild_player_client_packet import *
from .guild_open_client_packet import *
from .guild_kick_client_packet import *
from .guild_junk_client_packet import *
from .guild_info_type import *
from .guild_create_client_packet import *
from .guild_buy_client_packet import *
from .guild_agree_client_packet import *
from .guild_accept_client_packet import *
from .global_remove_client_packet import *
from .global_player_client_packet import *
from .global_open_client_packet import *
from .global_close_client_packet import *
from .file_type import *
from .face_player_client_packet import *
from .emote_report_client_packet import *
from .door_open_client_packet import *
from .dialog_reply import *
from .connection_ping_client_packet import *
from .connection_accept_client_packet import *
from .citizen_request_client_packet import *
from .citizen_reply_client_packet import *
from .citizen_remove_client_packet import *
from .citizen_open_client_packet import *
from .citizen_accept_client_packet import *
from .chest_take_client_packet import *
from .chest_open_client_packet import *
from .chest_add_client_packet import *
from .character_take_client_packet import *
from .character_request_client_packet import *
from .character_remove_client_packet import *
from .character_create_client_packet import *
from .chair_request_client_packet import *
from .byte_coords import *
from .book_request_client_packet import *
from .board_take_client_packet import *
from .board_remove_client_packet import *
from .board_open_client_packet import *
from .board_create_client_packet import *
from .barber_open_client_packet import *
from .barber_buy_client_packet import *
from .bank_take_client_packet import *
from .bank_open_client_packet import *
from .bank_add_client_packet import *
from .attack_use_client_packet import *
from .admin_interact_tell_client_packet import *
from .admin_interact_report_client_packet import *
from .account_request_client_packet import *
from .account_create_client_packet import *
from .account_agree_client_packet import *



from xml.etree.ElementTree import Element
from html import unescape


def get_instructions(element):
    return [
        child
        for child in element
        if isinstance(child, Element)
        and child.tag in ["field", "array", "length", "dummy", "switch", "chunked", "break"]
    ]


def get_comment(element):
    comment_element = next(
        (child for child in element if isinstance(child, Element) and child.tag == "comment"), None
    )

    if comment_element is not None:
        return get_text(comment_element)

    return None


def get_text(element):
    result = (element.text if element.text else "").strip()
    result = result.strip()

    for child in element:
        text = (child.tail if child.tail else "").strip()
        if text:
            if result:
                raise RuntimeError(f'Unexpected text content "{text}"')
            result = text

    return unescape(result) if result else None


def get_string_attribute(element, name, default_value=None):
    attribute_text = element.get(name)
    if attribute_text is None:
        return default_value
    return attribute_text


def get_int_attribute(element, name, default_value=0):
    attribute_text = element.get(name)
    if attribute_text is None:
        return default_value
    try:
        return int(attribute_text)
    except ValueError:
        raise ValueError(f'{name} attribute has an invalid integer value: {attribute_text.strip()}')


def get_boolean_attribute(element, name, default_value=False):
    attribute_text = element.get(name)
    if attribute_text is None:
        return default_value
    return attribute_text.lower() == "true"


def get_required_string_attribute(element, name):
    attribute_value = element.get(name)
    if attribute_value is None:
        raise ValueError(f'Required attribute "{name}" is missing.')
    return attribute_value


def get_required_int_attribute(element, name):
    attribute_value = element.get(name)
    if attribute_value is None:
        raise ValueError(f'Required attribute "{name}" is missing.')
    try:
        return int(attribute_value)
    except ValueError:
        raise ValueError(
            f'{name} attribute has an invalid integer value: {attribute_value.strip()}'
        )


def get_required_boolean_attribute(element, name):
    attribute_value = element.get(name)
    if attribute_value is None:
        raise ValueError(f'Required attribute "{name}" is missing.')
    return attribute_value.lower() == "true"



# Generated from the eo-protocol XML specification.
#
# This file should not be modified.
# Changes will be lost when code is regenerated.

from enum import IntEnum
from ....protocol_enum_meta import ProtocolEnumMeta

class WarpEffect(IntEnum, metaclass=ProtocolEnumMeta):
    """
    An effect that accompanies a player warp
    """
    None_ = 0
    """
    Does nothing
    """
    Scroll = 1
    """
    Plays the scroll sound effect
    """
    Admin = 2
    """
    Plays the admin warp sound effect and animation
    """



# Generated from the eo-protocol XML specification.
#
# This file should not be modified.
# Changes will be lost when code is regenerated.

from .quest_requirement_icon import QuestRequirementIcon
from ....serialization_error import SerializationError
from .....data.eo_writer import EoWriter
from .....data.eo_reader import EoReader

class QuestProgressEntry:
    """
    An entry in the Quest Progress window
    """
    _byte_size: int = 0
    _name: str
    _description: str
    _icon: QuestRequirementIcon
    _progress: int
    _target: int

    def __init__(self, *, name: str, description: str, icon: QuestRequirementIcon, progress: int, target: int):
        """
        Create a new instance of QuestProgressEntry.

        Args:
            name (str): 
            description (str): 
            icon (QuestRequirementIcon): 
            progress (int): (Value range is 0-64008.)
            target (int): (Value range is 0-64008.)
        """
        self._name = name
        self._description = description
        self._icon = icon
        self._progress = progress
        self._target = target

    @property
    def byte_size(self) -> int:
        """
        Returns the size of the data that this was deserialized from.

        Returns:
            int: The size of the data that this was deserialized from.
        """
        return self._byte_size

    @property
    def name(self) -> str:
        return self._name

    @property
    def description(self) -> str:
        return self._description

    @property
    def icon(self) -> QuestRequirementIcon:
        return self._icon

    @property
    def progress(self) -> int:
        return self._progress

    @property
    def target(self) -> int:
        return self._target

    @staticmethod
    def serialize(writer: EoWriter, data: "QuestProgressEntry") -> None:
        """
        Serializes an instance of `QuestProgressEntry` to the provided `EoWriter`.

        Args:
            writer (EoWriter): The writer that the data will be serialized to.
            data (QuestProgressEntry): The data to serialize.
        """
        old_string_sanitization_mode: bool = writer.string_sanitization_mode
        try:
            writer.string_sanitization_mode = True
            if data._name is None:
                raise SerializationError("name must be provided.")
            writer.add_string(data._name)
            writer.add_byte(0xFF)
            if data._description is None:
                raise SerializationError("description must be provided.")
            writer.add_string(data._description)
            writer.add_byte(0xFF)
            if data._icon is None:
                raise SerializationError("icon must be provided.")
            writer.add_short(int(data._icon))
            if data._progress is None:
                raise SerializationError("progress must be provided.")
            writer.add_short(data._progress)
            if data._target is None:
                raise SerializationError("target must be provided.")
            writer.add_short(data._target)
            writer.string_sanitization_mode = False
        finally:
            writer.string_sanitization_mode = old_string_sanitization_mode

    @staticmethod
    def deserialize(reader: EoReader) -> "QuestProgressEntry":
        """
        Deserializes an instance of `QuestProgressEntry` from the provided `EoReader`.

        Args:
            reader (EoReader): The writer that the data will be serialized to.

        Returns:
            QuestProgressEntry: The data to serialize.
        """
        old_chunked_reading_mode: bool = reader.chunked_reading_mode
        try:
            reader_start_position: int = reader.position
            reader.chunked_reading_mode = True
            name = reader.get_string()
            reader.next_chunk()
            description = reader.get_string()
            reader.next_chunk()
            icon = QuestRequirementIcon(reader.get_short())
            progress = reader.get_short()
            target = reader.get_short()
            reader.chunked_reading_mode = False
            result = QuestProgressEntry(name=name, description=description, icon=icon, progress=progress, target=target)
            result._byte_size = reader.position - reader_start_position
            return result
        finally:
            reader.chunked_reading_mode = old_chunked_reading_mode

    def __repr__(self):
        return f"QuestProgressEntry(byte_size={repr(self._byte_size)}, name={repr(self._name)}, description={repr(self._description)}, icon={repr(self._icon)}, progress={repr(self._progress)}, target={repr(self._target)})"



# Generated from the eo-protocol XML specification.
#
# This file should not be modified.
# Changes will be lost when code is regenerated.

from __future__ import annotations
from collections.abc import Iterable
from .drop_record import DropRecord
from ....serialization_error import SerializationError
from .....data.eo_writer import EoWriter
from .....data.eo_reader import EoReader

class DropNpcRecord:
    """
    Record of potential drops from an NPC
    """
    _byte_size: int = 0
    _npc_id: int
    _drops_count: int
    _drops: tuple[DropRecord, ...]

    def __init__(self, *, npc_id: int, drops: Iterable[DropRecord]):
        """
        Create a new instance of DropNpcRecord.

        Args:
            npc_id (int): (Value range is 0-64008.)
            drops (Iterable[DropRecord]): (Length must be 64008 or less.)
        """
        self._npc_id = npc_id
        self._drops = tuple(drops)
        self._drops_count = len(self._drops)

    @property
    def byte_size(self) -> int:
        """
        Returns the size of the data that this was deserialized from.

        Returns:
            int: The size of the data that this was deserialized from.
        """
        return self._byte_size

    @property
    def npc_id(self) -> int:
        return self._npc_id

    @property
    def drops(self) -> tuple[DropRecord, ...]:
        return self._drops

    @staticmethod
    def serialize(writer: EoWriter, data: "DropNpcRecord") -> None:
        """
        Serializes an instance of `DropNpcRecord` to the provided `EoWriter`.

        Args:
            writer (EoWriter): The writer that the data will be serialized to.
            data (DropNpcRecord): The data to serialize.
        """
        old_string_sanitization_mode: bool = writer.string_sanitization_mode
        try:
            if data._npc_id is None:
                raise SerializationError("npc_id must be provided.")
            writer.add_short(data._npc_id)
            if data._drops_count is None:
                raise SerializationError("drops_count must be provided.")
            writer.add_short(data._drops_count)
            if data._drops is None:
                raise SerializationError("drops must be provided.")
            if len(data._drops) > 64008:
                raise SerializationError(f"Expected length of drops to be 64008 or less, got {len(data._drops)}.")
            for i in range(data._drops_count):
                DropRecord.serialize(writer, data._drops[i])
        finally:
            writer.string_sanitization_mode = old_string_sanitization_mode

    @staticmethod
    def deserialize(reader: EoReader) -> "DropNpcRecord":
        """
        Deserializes an instance of `DropNpcRecord` from the provided `EoReader`.

        Args:
            reader (EoReader): The writer that the data will be serialized to.

        Returns:
            DropNpcRecord: The data to serialize.
        """
        old_chunked_reading_mode: bool = reader.chunked_reading_mode
        try:
            reader_start_position: int = reader.position
            npc_id = reader.get_short()
            drops_count = reader.get_short()
            drops = []
            for i in range(drops_count):
                drops.append(DropRecord.deserialize(reader))
            result = DropNpcRecord(npc_id=npc_id, drops=drops)
            result._byte_size = reader.position - reader_start_position
            return result
        finally:
            reader.chunked_reading_mode = old_chunked_reading_mode

    def __repr__(self):
        return f"DropNpcRecord(byte_size={repr(self._byte_size)}, npc_id={repr(self._npc_id)}, drops={repr(self._drops)})"



# Generated from the eo-protocol XML specification.
#
# This file should not be modified.
# Changes will be lost when code is regenerated.

from ....serialization_error import SerializationError
from .....data.eo_writer import EoWriter
from .....data.eo_reader import EoReader

class DialogQuestEntry:
    """
    An entry in the quest switcher
    """
    _byte_size: int = 0
    _quest_id: int
    _quest_name: str

    def __init__(self, *, quest_id: int, quest_name: str):
        """
        Create a new instance of DialogQuestEntry.

        Args:
            quest_id (int): (Value range is 0-64008.)
            quest_name (str): 
        """
        self._quest_id = quest_id
        self._quest_name = quest_name

    @property
    def byte_size(self) -> int:
        """
        Returns the size of the data that this was deserialized from.

        Returns:
            int: The size of the data that this was deserialized from.
        """
        return self._byte_size

    @property
    def quest_id(self) -> int:
        return self._quest_id

    @property
    def quest_name(self) -> str:
        return self._quest_name

    @staticmethod
    def serialize(writer: EoWriter, data: "DialogQuestEntry") -> None:
        """
        Serializes an instance of `DialogQuestEntry` to the provided `EoWriter`.

        Args:
            writer (EoWriter): The writer that the data will be serialized to.
            data (DialogQuestEntry): The data to serialize.
        """
        old_string_sanitization_mode: bool = writer.string_sanitization_mode
        try:
            if data._quest_id is None:
                raise SerializationError("quest_id must be provided.")
            writer.add_short(data._quest_id)
            if data._quest_name is None:
                raise SerializationError("quest_name must be provided.")
            writer.add_string(data._quest_name)
        finally:
            writer.string_sanitization_mode = old_string_sanitization_mode

    @staticmethod
    def deserialize(reader: EoReader) -> "DialogQuestEntry":
        """
        Deserializes an instance of `DialogQuestEntry` from the provided `EoReader`.

        Args:
            reader (EoReader): The writer that the data will be serialized to.

        Returns:
            DialogQuestEntry: The data to serialize.
        """
        old_chunked_reading_mode: bool = reader.chunked_reading_mode
        try:
            reader_start_position: int = reader.position
            quest_id = reader.get_short()
            quest_name = reader.get_string()
            result = DialogQuestEntry(quest_id=quest_id, quest_name=quest_name)
            result._byte_size = reader.position - reader_start_position
            return result
        finally:
            reader.chunked_reading_mode = old_chunked_reading_mode

    def __repr__(self):
        return f"DialogQuestEntry(byte_size={repr(self._byte_size)}, quest_id={repr(self._quest_id)}, quest_name={repr(self._quest_name)})"



# Generated from the eo-protocol XML specification.
#
# This file should not be modified.
# Changes will be lost when code is regenerated.

from enum import IntEnum
from ..protocol_enum_meta import ProtocolEnumMeta

class Direction(IntEnum, metaclass=ProtocolEnumMeta):
    """
    The direction a player or NPC is facing
    """
    Down = 0
    Left = 1
    Up = 2
    Right = 3



# Generated from the eo-protocol XML specification.
#
# This file should not be modified.
# Changes will be lost when code is regenerated.

from ..three_item import ThreeItem
from ..packet_family import PacketFamily
from ..packet_action import PacketAction
from ...coords import Coords
from ....serialization_error import SerializationError
from ....net.packet import Packet
from .....data.eo_writer import EoWriter
from .....data.eo_reader import EoReader

class LockerAddClientPacket(Packet):
    """
    Adding an item to a bank locker
    """
    _byte_size: int = 0
    _locker_coords: Coords
    _deposit_item: ThreeItem

    def __init__(self, *, locker_coords: Coords, deposit_item: ThreeItem):
        """
        Create a new instance of LockerAddClientPacket.

        Args:
            locker_coords (Coords): 
            deposit_item (ThreeItem): 
        """
        self._locker_coords = locker_coords
        self._deposit_item = deposit_item

    @property
    def byte_size(self) -> int:
        """
        Returns the size of the data that this was deserialized from.

        Returns:
            int: The size of the data that this was deserialized from.
        """
        return self._byte_size

    @property
    def locker_coords(self) -> Coords:
        return self._locker_coords

    @property
    def deposit_item(self) -> ThreeItem:
        return self._deposit_item

    @staticmethod
    def family() -> PacketFamily:
        """
        Returns the packet family associated with this packet.

        Returns:
            PacketFamily: The packet family associated with this packet.
        """
        return PacketFamily.Locker

    @staticmethod
    def action() -> PacketAction:
        """
        Returns the packet action associated with this packet.

        Returns:
            PacketAction: The packet action associated with this packet.
        """
        return PacketAction.Add

    def write(self, writer):
        """
        Serializes and writes this packet to the provided EoWriter.

        Args:
            writer (EoWriter): the writer that this packet will be written to.
        """
        LockerAddClientPacket.serialize(writer, self)

    @staticmethod
    def serialize(writer: EoWriter, data: "LockerAddClientPacket") -> None:
        """
        Serializes an instance of `LockerAddClientPacket` to the provided `EoWriter`.

        Args:
            writer (EoWriter): The writer that the data will be serialized to.
            data (LockerAddClientPacket): The data to serialize.
        """
        old_string_sanitization_mode: bool = writer.string_sanitization_mode
        try:
            if data._locker_coords is None:
                raise SerializationError("locker_coords must be provided.")
            Coords.serialize(writer, data._locker_coords)
            if data._deposit_item is None:
                raise SerializationError("deposit_item must be provided.")
            ThreeItem.serialize(writer, data._deposit_item)
        finally:
            writer.string_sanitization_mode = old_string_sanitization_mode

    @staticmethod
    def deserialize(reader: EoReader) -> "LockerAddClientPacket":
        """
        Deserializes an instance of `LockerAddClientPacket` from the provided `EoReader`.

        Args:
            reader (EoReader): The writer that the data will be serialized to.

        Returns:
            LockerAddClientPacket: The data to serialize.
        """
        old_chunked_reading_mode: bool = reader.chunked_reading_mode
        try:
            reader_start_position: int = reader.position
            locker_coords = Coords.deserialize(reader)
            deposit_item = ThreeItem.deserialize(reader)
            result = LockerAddClientPacket(locker_coords=locker_coords, deposit_item=deposit_item)
            result._byte_size = reader.position - reader_start_position
            return result
        finally:
            reader.chunked_reading_mode = old_chunked_reading_mode

    def __repr__(self):
        return f"LockerAddClientPacket(byte_size={repr(self._byte_size)}, locker_coords={repr(self._locker_coords)}, deposit_item={repr(self._deposit_item)})"



"""
Utilities to read and write EO data types.
"""

from .eo_numeric_limits import *

from .number_encoding_utils import *
from .string_encoding_utils import *

from .eo_reader import *
from .eo_writer import *



# Generated from the eo-protocol XML specification.
#
# This file should not be modified.
# Changes will be lost when code is regenerated.

from typing import Union
from .dialog_entry_type import DialogEntryType
from ....serialization_error import SerializationError
from .....data.eo_writer import EoWriter
from .....data.eo_reader import EoReader

class DialogEntry:
    """
    An entry in a quest dialog
    """
    _byte_size: int = 0
    _entry_type: DialogEntryType
    _entry_type_data: 'DialogEntry.EntryTypeData'
    _line: str

    def __init__(self, *, entry_type: DialogEntryType, entry_type_data: 'DialogEntry.EntryTypeData' = None, line: str):
        """
        Create a new instance of DialogEntry.

        Args:
            entry_type (DialogEntryType): 
            entry_type_data (DialogEntry.EntryTypeData): Data associated with the `entry_type` field.
            line (str): 
        """
        self._entry_type = entry_type
        self._entry_type_data = entry_type_data
        self._line = line

    @property
    def byte_size(self) -> int:
        """
        Returns the size of the data that this was deserialized from.

        Returns:
            int: The size of the data that this was deserialized from.
        """
        return self._byte_size

    @property
    def entry_type(self) -> DialogEntryType:
        return self._entry_type

    @property
    def entry_type_data(self) -> 'DialogEntry.EntryTypeData':
        """
        DialogEntry.EntryTypeData: Data associated with the `entry_type` field.
        """
        return self._entry_type_data

    @property
    def line(self) -> str:
        return self._line

    @staticmethod
    def serialize(writer: EoWriter, data: "DialogEntry") -> None:
        """
        Serializes an instance of `DialogEntry` to the provided `EoWriter`.

        Args:
            writer (EoWriter): The writer that the data will be serialized to.
            data (DialogEntry): The data to serialize.
        """
        old_string_sanitization_mode: bool = writer.string_sanitization_mode
        try:
            if data._entry_type is None:
                raise SerializationError("entry_type must be provided.")
            writer.add_short(int(data._entry_type))
            if data._entry_type == DialogEntryType.Link:
                if not isinstance(data._entry_type_data, DialogEntry.EntryTypeDataLink):
                    raise SerializationError("Expected entry_type_data to be type DialogEntry.EntryTypeDataLink for entry_type " + DialogEntryType(data._entry_type).name + ".")
                DialogEntry.EntryTypeDataLink.serialize(writer, data._entry_type_data)
            if data._line is None:
                raise SerializationError("line must be provided.")
            writer.add_string(data._line)
        finally:
            writer.string_sanitization_mode = old_string_sanitization_mode

    @staticmethod
    def deserialize(reader: EoReader) -> "DialogEntry":
        """
        Deserializes an instance of `DialogEntry` from the provided `EoReader`.

        Args:
            reader (EoReader): The writer that the data will be serialized to.

        Returns:
            DialogEntry: The data to serialize.
        """
        old_chunked_reading_mode: bool = reader.chunked_reading_mode
        try:
            reader_start_position: int = reader.position
            entry_type = DialogEntryType(reader.get_short())
            entry_type_data: DialogEntry.EntryTypeData = None
            if entry_type == DialogEntryType.Link:
                entry_type_data = DialogEntry.EntryTypeDataLink.deserialize(reader)
            line = reader.get_string()
            result = DialogEntry(entry_type=entry_type, entry_type_data=entry_type_data, line=line)
            result._byte_size = reader.position - reader_start_position
            return result
        finally:
            reader.chunked_reading_mode = old_chunked_reading_mode

    def __repr__(self):
        return f"DialogEntry(byte_size={repr(self._byte_size)}, entry_type={repr(self._entry_type)}, entry_type_data={repr(self._entry_type_data)}, line={repr(self._line)})"

    EntryTypeData = Union['DialogEntry.EntryTypeDataLink', None]
    """
    Data associated with different values of the `entry_type` field.
    """

    class EntryTypeDataLink:
        """
        Data associated with entry_type value DialogEntryType.Link
        """
        _byte_size: int = 0
        _link_id: int

        def __init__(self, *, link_id: int):
            """
            Create a new instance of DialogEntry.EntryTypeDataLink.

            Args:
                link_id (int): (Value range is 0-64008.)
            """
            self._link_id = link_id

        @property
        def byte_size(self) -> int:
            """
            Returns the size of the data that this was deserialized from.

            Returns:
                int: The size of the data that this was deserialized from.
            """
            return self._byte_size

        @property
        def link_id(self) -> int:
            return self._link_id

        @staticmethod
        def serialize(writer: EoWriter, data: "DialogEntry.EntryTypeDataLink") -> None:
            """
            Serializes an instance of `DialogEntry.EntryTypeDataLink` to the provided `EoWriter`.

            Args:
                writer (EoWriter): The writer that the data will be serialized to.
                data (DialogEntry.EntryTypeDataLink): The data to serialize.
            """
            old_string_sanitization_mode: bool = writer.string_sanitization_mode
            try:
                if data._link_id is None:
                    raise SerializationError("link_id must be provided.")
                writer.add_short(data._link_id)
            finally:
                writer.string_sanitization_mode = old_string_sanitization_mode

        @staticmethod
        def deserialize(reader: EoReader) -> "DialogEntry.EntryTypeDataLink":
            """
            Deserializes an instance of `DialogEntry.EntryTypeDataLink` from the provided `EoReader`.

            Args:
                reader (EoReader): The writer that the data will be serialized to.

            Returns:
                DialogEntry.EntryTypeDataLink: The data to serialize.
            """
            old_chunked_reading_mode: bool = reader.chunked_reading_mode
            try:
                reader_start_position: int = reader.position
                link_id = reader.get_short()
                result = DialogEntry.EntryTypeDataLink(link_id=link_id)
                result._byte_size = reader.position - reader_start_position
                return result
            finally:
                reader.chunked_reading_mode = old_chunked_reading_mode

        def __repr__(self):
            return f"DialogEntry.EntryTypeDataLink(byte_size={repr(self._byte_size)}, link_id={repr(self._link_id)})"



import pytest
from collections import namedtuple
from eolib.encrypt.encryption_utils import interleave, deinterleave, flip_msb, swap_multiples

TCase = namedtuple("TCase", ["string", "expected"])


def idfn(test_case: TCase) -> str:
    return test_case.string


@pytest.mark.parametrize(
    "test_case",
    [
        TCase("Hello, World!", "H!edlllroo,W "),
        TCase(
            "We're ¼ of the way there, so ¾ is remaining.",
            "W.eg'nrien i¼a moefr  tshie  ¾w aoys  t,heer",
        ),
        TCase("64² = 4096", "6649²0 4= "),
        TCase("© FÒÖ BÃR BÅZ 2014", "©4 1F0Ò2Ö  ZBÅÃBR "),
        TCase("Öxxö Xööx \"Lëïth Säë\" - \"Ÿ\"", "Ö\"xŸx\"ö  -X ö\"öëxä S\" Lhëtï"),
        TCase("Padded with 0xFFÿÿÿÿÿÿÿÿ", "Pÿaÿdÿdÿeÿdÿ ÿwÿiFtFhx 0"),
        TCase(
            "This string contains NUL\0 (value 0) and a € (value 128)",
            "T)h8i2s1  seturlianvg(  c€o nat adinnas  )N0U Le\0u l(av",
        ),
    ],
    ids=idfn,
)
def test_interleave(test_case: TCase):
    bytes_data = bytearray(test_case.string, 'windows-1252')
    interleave(bytes_data)
    result = bytes_data.decode('windows-1252')
    assert result == test_case.expected


@pytest.mark.parametrize(
    "test_case",
    [
        TCase("Hello, World!", "Hlo ol!drW,le"),
        TCase(
            "We're ¼ of the way there, so ¾ is remaining.",
            "W'e¼o h a hr,s  srmiig.nnae i¾o eetywetf  re",
        ),
        TCase("64² = 4096", "6²=4960  4"),
        TCase("© FÒÖ BÃR BÅZ 2014", "©FÖBRBZ2140 Å Ã Ò "),
        TCase("Öxxö Xööx \"Lëïth Säë\" - \"Ÿ\"", "Öx öx\"ët ä\"-\"\"Ÿ  ëShïL öXöx"),
        TCase("Padded with 0xFFÿÿÿÿÿÿÿÿ", "Pde ih0FÿÿÿÿÿÿÿÿFx twdda"),
        TCase(
            "This string contains NUL\0 (value 0) and a € (value 128)",
            "Ti tigcnan U\0(au )ada€(au 2)81elv   n 0elv LNsito nrssh",
        ),
    ],
    ids=idfn,
)
def test_deinterleave(test_case: TCase):
    bytes_data = bytearray(test_case.string, 'windows-1252')
    deinterleave(bytes_data)
    result = bytes_data.decode('windows-1252')
    assert result == test_case.expected


@pytest.mark.parametrize(
    "test_case",
    [
        TCase("Hello, World!", "Èåììï¬\u00a0×ïòìä¡"),
        TCase(
            "We're ¼ of the way there, so ¾ is remaining.",
            "×å§òå\u00a0<\u00a0ïæ\u00a0ôèå\u00a0÷áù\u00a0ôèåòå¬\u00a0óï\u00a0>\u00a0éó\u00a0òåíáéîéîç®",
        ),
        TCase("64² = 4096", "¶´2\u00a0½\u00a0´°¹¶"),
        TCase("© FÒÖ BÃR BÅZ 2014", ")\u00a0ÆRV\u00a0ÂCÒ\u00a0ÂEÚ\u00a0²°±´"),
        TCase(
            "Öxxö Xööx \"Lëïth Säë\" - \"Ÿ\"",
            "Vøøv\u00a0Øvvø\u00a0¢Ìkoôè\u00a0Ódk¢\u00a0\u00ad\u00a0¢\u001f¢",
        ),
        TCase(
            "Padded with 0xFFÿÿÿÿÿÿÿÿ",
            "Ðáääåä\u00a0÷éôè\u00a0°øÆÆ\u007f\u007f\u007f\u007f\u007f\u007f\u007f\u007f",
        ),
        TCase(
            "This string contains NUL\0 (value 0) and a € (value 128)",
            "Ôèéó\u00a0óôòéîç\u00a0ãïîôáéîó\u00a0ÎÕÌ\0\u00a0¨öáìõå\u00a0°©\u00a0áîä\u00a0á\u00a0€\u00a0¨öáìõå\u00a0±²¸©",
        ),
    ],
    ids=idfn,
)
def test_flip_msb(test_case: TCase):
    bytes_data = bytearray(test_case.string, 'windows-1252')
    flip_msb(bytes_data)
    result = bytes_data.decode('windows-1252')
    assert result == test_case.expected


@pytest.mark.parametrize(
    "test_case",
    [
        TCase("Hello, World!", "Heoll, lroWd!"),
        TCase(
            "We're ¼ of the way there, so ¾ is remaining.",
            "Wer'e ¼ fo the way there, so ¾ is remaining.",
        ),
        TCase("64² = 4096", "64² = 4690"),
        TCase("© FÒÖ BÃR BÅZ 2014", "© FÒÖ ÃBR BÅZ 2014"),
        TCase("Öxxö Xööx \"Lëïth Säë\" - \"Ÿ\"", "Ööxx Xxöö \"Lëïth Säë\" - \"Ÿ\""),
        TCase("Padded with 0xFFÿÿÿÿÿÿÿÿ", "Padded with x0FFÿÿÿÿÿÿÿÿ"),
        TCase(
            "This string contains NUL\0 (value 0) and a € (value 128)",
            "This stirng ocntains NUL\0 (vaule 0) and a € (vaule 128)",
        ),
    ],
    ids=idfn,
)
def test_swap_multiples(test_case: TCase):
    bytes_data = bytearray(test_case.string, 'windows-1252')
    swap_multiples(bytes_data, 3)
    result = bytes_data.decode('windows-1252')
    assert result == test_case.expected


def test_swap_multiples_with_zero_multiple_should_not_change_data():
    string = "Hello, World!"
    bytes_data = bytearray(string, 'windows-1252')
    swap_multiples(bytes_data, 0)
    result = bytes_data.decode('windows-1252')
    assert result == string


def test_swap_multiples_with_negative_multiple_should_throw():
    with pytest.raises(ValueError):
        swap_multiples(bytearray([1, 2, 3, 4, 5]), -1)



# Generated from the eo-protocol XML specification.
#
# This file should not be modified.
# Changes will be lost when code is regenerated.

from enum import IntEnum
from ...protocol_enum_meta import ProtocolEnumMeta

class MapType(IntEnum, metaclass=ProtocolEnumMeta):
    Normal = 0
    Pk = 3



# Generated from the eo-protocol XML specification.
#
# This file should not be modified.
# Changes will be lost when code is regenerated.

from __future__ import annotations
from collections.abc import Iterable
from .map_warp_row_tile import MapWarpRowTile
from ...serialization_error import SerializationError
from ....data.eo_writer import EoWriter
from ....data.eo_reader import EoReader

class MapWarpRow:
    """
    A row of warp entities
    """
    _byte_size: int = 0
    _y: int
    _tiles_count: int
    _tiles: tuple[MapWarpRowTile, ...]

    def __init__(self, *, y: int, tiles: Iterable[MapWarpRowTile]):
        """
        Create a new instance of MapWarpRow.

        Args:
            y (int): (Value range is 0-252.)
            tiles (Iterable[MapWarpRowTile]): (Length must be 252 or less.)
        """
        self._y = y
        self._tiles = tuple(tiles)
        self._tiles_count = len(self._tiles)

    @property
    def byte_size(self) -> int:
        """
        Returns the size of the data that this was deserialized from.

        Returns:
            int: The size of the data that this was deserialized from.
        """
        return self._byte_size

    @property
    def y(self) -> int:
        return self._y

    @property
    def tiles(self) -> tuple[MapWarpRowTile, ...]:
        return self._tiles

    @staticmethod
    def serialize(writer: EoWriter, data: "MapWarpRow") -> None:
        """
        Serializes an instance of `MapWarpRow` to the provided `EoWriter`.

        Args:
            writer (EoWriter): The writer that the data will be serialized to.
            data (MapWarpRow): The data to serialize.
        """
        old_string_sanitization_mode: bool = writer.string_sanitization_mode
        try:
            if data._y is None:
                raise SerializationError("y must be provided.")
            writer.add_char(data._y)
            if data._tiles_count is None:
                raise SerializationError("tiles_count must be provided.")
            writer.add_char(data._tiles_count)
            if data._tiles is None:
                raise SerializationError("tiles must be provided.")
            if len(data._tiles) > 252:
                raise SerializationError(f"Expected length of tiles to be 252 or less, got {len(data._tiles)}.")
            for i in range(data._tiles_count):
                MapWarpRowTile.serialize(writer, data._tiles[i])
        finally:
            writer.string_sanitization_mode = old_string_sanitization_mode

    @staticmethod
    def deserialize(reader: EoReader) -> "MapWarpRow":
        """
        Deserializes an instance of `MapWarpRow` from the provided `EoReader`.

        Args:
            reader (EoReader): The writer that the data will be serialized to.

        Returns:
            MapWarpRow: The data to serialize.
        """
        old_chunked_reading_mode: bool = reader.chunked_reading_mode
        try:
            reader_start_position: int = reader.position
            y = reader.get_char()
            tiles_count = reader.get_char()
            tiles = []
            for i in range(tiles_count):
                tiles.append(MapWarpRowTile.deserialize(reader))
            result = MapWarpRow(y=y, tiles=tiles)
            result._byte_size = reader.position - reader_start_position
            return result
        finally:
            reader.chunked_reading_mode = old_chunked_reading_mode

    def __repr__(self):
        return f"MapWarpRow(byte_size={repr(self._byte_size)}, y={repr(self._y)}, tiles={repr(self._tiles)})"
